@@ -11,7 +11,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tcl.h 1.324 97/08/07 10:26:49
+ * SCCS: @(#) tcl.h 1.24 98/08/06 12:10:41
  */
 
 #ifndef _TCL
@@ -21,11 +21,11 @@
  * When version numbers change here, must also go into the following files
  * and update the version numbers:
  *
- * library/init.tcl
+ * README
+ * library/init.tcl	(only if major.minor changes, not patchlevel)
  * unix/configure.in
- * unix/pkginfo
- * win/makefile.bc
- * win/makefile.vc
+ * win/makefile.bc	(only if major.minor changes, not patchlevel)
+ * win/makefile.vc	(only if major.minor changes, not patchlevel)
  *
  * The release level should be  0 for alpha, 1 for beta, and 2 for
  * final/patch.  The release serial value is the number that follows the
@@ -38,10 +38,10 @@
 #define TCL_MAJOR_VERSION   8
 #define TCL_MINOR_VERSION   0
 #define TCL_RELEASE_LEVEL   2
-#define TCL_RELEASE_SERIAL  0
+#define TCL_RELEASE_SERIAL  3
 
 #define TCL_VERSION	    "8.0"
-#define TCL_PATCH_LEVEL	    "8.0"
+#define TCL_PATCH_LEVEL	    "8.0.3+"
 
 /*
  * The following definitions set up the proper options for Windows
@@ -70,10 +70,6 @@
 #   ifndef USE_TCLALLOC
 #	define USE_TCLALLOC 1
 #   endif
-#   ifndef STRINGIFY
-#	define STRINGIFY(x)	    STRINGIFY1(x)
-#	define STRINGIFY1(x)	    #x
-#   endif
 #endif /* __WIN32__ */
 
 /*
@@ -93,6 +89,34 @@
 #   endif
 #endif
 
+/*
+ * Utility macros: STRINGIFY takes an argument and wraps it in "" (double
+ * quotation marks), JOIN joins two arguments.
+ */
+
+#define VERBATIM(x) x
+#ifdef _MSC_VER
+# define STRINGIFY(x) STRINGIFY1(x)
+# define STRINGIFY1(x) #x
+# define JOIN(a,b) JOIN1(a,b)
+# define JOIN1(a,b) a##b
+#else
+# ifdef RESOURCE_INCLUDED
+#  define STRINGIFY(x) STRINGIFY1(x)
+#  define STRINGIFY1(x) #x
+#  define JOIN(a,b) JOIN1(a,b)
+#  define JOIN1(a,b) a##b
+# else
+#  ifdef __STDC__
+#   define STRINGIFY(x) #x
+#   define JOIN(a,b) a##b
+#  else
+#   define STRINGIFY(x) "x"
+#   define JOIN(a,b) VERBATIM(a)VERBATIM(b)
+#  endif
+# endif
+#endif
+
 /* 
  * A special definition used to allow this header file to be included 
  * in resource files so that they can get obtain version information from
@@ -132,6 +156,45 @@
 	(va_start(list), va_arg(list, type))
 #endif
 
+/*
+ * Macros used to declare a function to be exported by a DLL.
+ * Used by Windows, maps to no-op declarations on non-Windows systems.
+ * The default build on windows is for a DLL, which causes the DLLIMPORT
+ * and DLLEXPORT macros to be nonempty. To build a static library, the
+ * macro STATIC_BUILD should be defined.
+ * The support follows the convention that a macro called BUILD_xxxx, where
+ * xxxx is the name of a library we are building, is set on the compile line
+ * for sources that are to be placed in the library. See BUILD_tcl in this
+ * file for an example of how the macro is to be used.
+ */
+
+#ifdef __WIN32__
+# ifdef STATIC_BUILD
+#  define DLLIMPORT
+#  define DLLEXPORT
+# else
+#  ifdef _MSC_VER
+#   define DLLIMPORT __declspec(dllimport)
+#   define DLLEXPORT __declspec(dllexport)
+#  else
+#   define DLLIMPORT
+#   define DLLEXPORT
+#  endif
+# endif
+#else
+# define DLLIMPORT
+# define DLLEXPORT
+#endif
+
+#ifdef TCL_STORAGE_CLASS
+# undef TCL_STORAGE_CLASS
+#endif
+#ifdef BUILD_tcl
+# define TCL_STORAGE_CLASS DLLEXPORT
+#else
+# define TCL_STORAGE_CLASS DLLIMPORT
+#endif
+
 /*
  * Definitions that allow this header file to be used either with or
  * without ANSI C features like function prototypes.
@@ -150,9 +213,9 @@
 #endif
 
 #ifdef __cplusplus
-#   define EXTERN extern "C"
+#   define EXTERN extern "C" TCL_STORAGE_CLASS
 #else
-#   define EXTERN extern
+#   define EXTERN extern TCL_STORAGE_CLASS
 #endif
 
 /*
@@ -443,7 +506,7 @@ EXTERN Tcl_Obj *	Tcl_NewListObj _ANSI_ARGS_((int objc,
 			    Tcl_Obj *CONST objv[]));
 EXTERN Tcl_Obj *	Tcl_NewLongObj _ANSI_ARGS_((long longValue));
 EXTERN Tcl_Obj *	Tcl_NewObj _ANSI_ARGS_((void));
-EXTERN Tcl_Obj *	Tcl_NewStringObj _ANSI_ARGS_((char *bytes,
+EXTERN Tcl_Obj *	Tcl_NewStringObj _ANSI_ARGS_((CONST char *bytes,
 			    int length));
 
 #ifdef TCL_MEM_DEBUG
@@ -591,6 +654,14 @@ typedef struct Tcl_DString {
 #define TCL_MAX_PREC 17
 #define TCL_DOUBLE_SPACE (TCL_MAX_PREC+10)
 
+/*
+ * Definition for a number of bytes of buffer space sufficient to hold the
+ * string representation of an integer in base 10 (assuming the existence
+ * of 64-bit integers).
+ */
+
+#define TCL_INTEGER_SPACE	24
+
 /*
  * Flag that may be passed to Tcl_ConvertElement to force it not to
  * output braces (careful!  if you change this flag be sure to change
@@ -660,6 +731,10 @@ EXTERN char *		Tcl_Alloc _ANSI_ARGS_((unsigned int size));
 EXTERN void		Tcl_Free _ANSI_ARGS_((char *ptr));
 EXTERN char *		Tcl_Realloc _ANSI_ARGS_((char *ptr,
 			    unsigned int size));
+EXTERN void		Tcl_InitMemory _ANSI_ARGS_((Tcl_Interp *interp));
+EXTERN int		Tcl_DumpActiveMemory _ANSI_ARGS_((char *fileName));
+EXTERN void		Tcl_ValidateAllMemory _ANSI_ARGS_((char *file,
+			    int line));
 
 #ifdef TCL_MEM_DEBUG
 
@@ -670,10 +745,6 @@ EXTERN char *		Tcl_Realloc _ANSI_ARGS_((char *ptr,
 #  define ckfree(x)  Tcl_DbCkfree(x, __FILE__, __LINE__)
 #  define ckrealloc(x,y) Tcl_DbCkrealloc((x), (y),__FILE__, __LINE__)
 
-EXTERN int		Tcl_DumpActiveMemory _ANSI_ARGS_((char *fileName));
-EXTERN void		Tcl_ValidateAllMemory _ANSI_ARGS_((char *file,
-			    int line));
-
 #else
 
 #  if USE_TCLALLOC
@@ -685,6 +756,7 @@ EXTERN void		Tcl_ValidateAllMemory _ANSI_ARGS_((char *file,
 #     define ckfree(x)  free(x)
 #     define ckrealloc(x,y) realloc(x,y)
 #  endif
+#  define Tcl_InitMemory(x)
 #  define Tcl_DumpActiveMemory(x)
 #  define Tcl_ValidateAllMemory(x,y)
 
@@ -868,6 +940,12 @@ typedef struct Tcl_Time {
 #define TCL_READABLE	(1<<1)
 #define TCL_WRITABLE	(1<<2)
 #define TCL_EXCEPTION	(1<<3)
+#define TCL_ACTIVE	0
+
+typedef struct Tcl_StaticFile {
+    CONST char *name;
+    char **content;
+} Tcl_StaticFile;
 
 /*
  * Flag values to pass to Tcl_OpenCommandChannel to indicate the
@@ -917,6 +995,15 @@ typedef enum Tcl_EolTranslation {
     TCL_TRANSLATE_CRLF			/* Eol == \r\n. */
 } Tcl_EolTranslation;
 
+/*
+ * Definitions for different byteorders.
+ */
+
+typedef int Tcl_ByteOrder;
+
+#define TCL_BIGENDIAN 0
+#define TCL_SMALLENDIAN 1
+
 /*
  * struct Tcl_ChannelType:
  *
@@ -949,6 +1036,7 @@ typedef struct Tcl_ChannelType {
     Tcl_DriverGetHandleProc *getHandleProc;
 					/* Get an OS handle from the channel
                                          * or NULL if not supported. */
+    VOID *reserved;			/* reserved for future expansion */
 } Tcl_ChannelType;
 
 /*
@@ -976,21 +1064,21 @@ typedef enum Tcl_PathType {
  */
 
 EXTERN void		Tcl_AddErrorInfo _ANSI_ARGS_((Tcl_Interp *interp,
-			    char *message));
+			    CONST char *message));
 EXTERN void		Tcl_AddObjErrorInfo _ANSI_ARGS_((Tcl_Interp *interp,
-			    char *message, int length));
+			    CONST char *message, int length));
 EXTERN void		Tcl_AllowExceptions _ANSI_ARGS_((Tcl_Interp *interp));
 EXTERN int		Tcl_AppendAllObjTypes _ANSI_ARGS_((
 			    Tcl_Interp *interp, Tcl_Obj *objPtr));
 EXTERN void		Tcl_AppendElement _ANSI_ARGS_((Tcl_Interp *interp,
-			    char *string));
+			    CONST char *string));
 EXTERN void		Tcl_AppendResult _ANSI_ARGS_(
 			    TCL_VARARGS(Tcl_Interp *,interp));
 EXTERN void		Tcl_AppendToObj _ANSI_ARGS_((Tcl_Obj *objPtr,
 			    char *bytes, int length));
 EXTERN void		Tcl_AppendStringsToObj _ANSI_ARGS_(
 			    TCL_VARARGS(Tcl_Obj *,interp));
-EXTERN int		Tcl_AppInit _ANSI_ARGS_((Tcl_Interp *interp));
+extern int		Tcl_AppInit _ANSI_ARGS_((Tcl_Interp *interp));
 EXTERN Tcl_AsyncHandler	Tcl_AsyncCreate _ANSI_ARGS_((Tcl_AsyncProc *proc,
 			    ClientData clientData));
 EXTERN void		Tcl_AsyncDelete _ANSI_ARGS_((Tcl_AsyncHandler async));
@@ -1088,8 +1176,10 @@ EXTERN Tcl_Obj *	Tcl_DbNewListObj _ANSI_ARGS_((int objc,
 EXTERN Tcl_Obj *	Tcl_DbNewLongObj _ANSI_ARGS_((long longValue,
                             char *file, int line));
 EXTERN Tcl_Obj *	Tcl_DbNewObj _ANSI_ARGS_((char *file, int line));
-EXTERN Tcl_Obj *	Tcl_DbNewStringObj _ANSI_ARGS_((char *bytes,
+EXTERN Tcl_Obj *	Tcl_DbNewStringObj _ANSI_ARGS_((CONST char *bytes,
 			    int length, char *file, int line));
+EXTERN void		Tcl_DefineStaticFile _ANSI_ARGS_((
+			    Tcl_StaticFile *table));
 EXTERN void		Tcl_DeleteAssocData _ANSI_ARGS_((Tcl_Interp *interp,
                             char *name));
 EXTERN int		Tcl_DeleteCommand _ANSI_ARGS_((Tcl_Interp *interp,
@@ -1175,7 +1265,9 @@ EXTERN int		Tcl_ExprObj _ANSI_ARGS_((Tcl_Interp *interp,
 EXTERN int		Tcl_ExprString _ANSI_ARGS_((Tcl_Interp *interp,
 			    char *string));
 EXTERN void		Tcl_Finalize _ANSI_ARGS_((void));
-EXTERN void		Tcl_FindExecutable _ANSI_ARGS_((char *argv0));
+EXTERN void		Tcl_FindExecutable _ANSI_ARGS_((CONST char *argv0));
+EXTERN VOID *		Tcl_FindPackageHandle _ANSI_ARGS_((
+			    CONST char *packageName));
 EXTERN Tcl_HashEntry *	Tcl_FirstHashEntry _ANSI_ARGS_((
 			    Tcl_HashTable *tablePtr,
 			    Tcl_HashSearch *searchPtr));
@@ -1201,6 +1293,12 @@ EXTERN Tcl_Channel	Tcl_GetChannel _ANSI_ARGS_((Tcl_Interp *interp,
 	        	    char *chanName, int *modePtr));
 EXTERN int		Tcl_GetChannelBufferSize _ANSI_ARGS_((
     			    Tcl_Channel chan));
+  /* Andreas Kupries <a.kupries@westend.com>, 05/31/1997.
+   * Support of Tcl-Trf (binio).
+   */
+EXTERN int		Tcl_GetChannelByteorder _ANSI_ARGS_((
+			    Tcl_Channel chan));
+EXTERN int		Tcl_GetHostByteorder _ANSI_ARGS_((void));
 EXTERN int		Tcl_GetChannelHandle _ANSI_ARGS_((Tcl_Channel chan,
 	        	    int direction, ClientData *handlePtr));
 EXTERN ClientData	Tcl_GetChannelInstanceData _ANSI_ARGS_((
@@ -1236,7 +1334,10 @@ EXTERN int		Tcl_GetIntFromObj _ANSI_ARGS_((Tcl_Interp *interp,
 EXTERN int		Tcl_GetLongFromObj _ANSI_ARGS_((Tcl_Interp *interp,
 			    Tcl_Obj *objPtr, long *longPtr));
 EXTERN Tcl_Interp *	Tcl_GetMaster _ANSI_ARGS_((Tcl_Interp *interp));
+EXTERN CONST char *	Tcl_GetNameOfExecutable _ANSI_ARGS_((void));
 EXTERN Tcl_Obj *	Tcl_GetObjResult _ANSI_ARGS_((Tcl_Interp *interp));
+EXTERN Tcl_Obj *	Tcl_GetObjVar2 _ANSI_ARGS_((Tcl_Interp *interp,
+			    char *part1, char *part2, int flags));
 EXTERN Tcl_ObjType *	Tcl_GetObjType _ANSI_ARGS_((char *typeName));
 EXTERN int		Tcl_GetOpenFile _ANSI_ARGS_((Tcl_Interp *interp,
 			    char *string, int write, int checkUsage,
@@ -1251,7 +1352,9 @@ EXTERN int		Tcl_GetsObj _ANSI_ARGS_((Tcl_Channel chan,
 EXTERN int		Tcl_GetServiceMode _ANSI_ARGS_((void));
 EXTERN Tcl_Interp *	Tcl_GetSlave _ANSI_ARGS_((Tcl_Interp *interp,
 			    char *slaveName));
+EXTERN char **		Tcl_GetStaticFile _ANSI_ARGS_((CONST char *rsrcName));
 EXTERN Tcl_Channel	Tcl_GetStdChannel _ANSI_ARGS_((int type));
+EXTERN char *		Tcl_GetString _ANSI_ARGS_((Tcl_Obj *objPtr));
 EXTERN char *		Tcl_GetStringFromObj _ANSI_ARGS_((Tcl_Obj *objPtr,
 			    int *lengthPtr));
 EXTERN char *		Tcl_GetStringResult _ANSI_ARGS_((Tcl_Interp *interp));
@@ -1267,9 +1370,10 @@ EXTERN char *		Tcl_HashStats _ANSI_ARGS_((Tcl_HashTable *tablePtr));
 EXTERN int		Tcl_HideCommand _ANSI_ARGS_((Tcl_Interp *interp,
 		            char *cmdName, char *hiddenCmdToken));
 EXTERN int		Tcl_Init _ANSI_ARGS_((Tcl_Interp *interp));
+EXTERN int		Tcl_InitStandAlone _ANSI_ARGS_((Tcl_Interp *interp));
 EXTERN void		Tcl_InitHashTable _ANSI_ARGS_((Tcl_HashTable *tablePtr,
 			    int keyType));
-EXTERN void		Tcl_InitMemory _ANSI_ARGS_((Tcl_Interp *interp));
+EXTERN ClientData	Tcl_InitNotifier _ANSI_ARGS_((void));
 EXTERN int		Tcl_InputBlocked _ANSI_ARGS_((Tcl_Channel chan));
 EXTERN int		Tcl_InputBuffered _ANSI_ARGS_((Tcl_Channel chan));
 EXTERN int		Tcl_InterpDeleted _ANSI_ARGS_((Tcl_Interp *interp));
@@ -1299,6 +1403,7 @@ EXTERN int		Tcl_ListObjReplace _ANSI_ARGS_((Tcl_Interp *interp,
 			    int objc, Tcl_Obj *CONST objv[]));
 EXTERN void		Tcl_Main _ANSI_ARGS_((int argc, char **argv,
 			    Tcl_AppInitProc *appInitProc));
+EXTERN void		Tcl_MainLoop _ANSI_ARGS_((void));
 EXTERN Tcl_Channel	Tcl_MakeFileChannel _ANSI_ARGS_((ClientData handle,
 			    int mode));
 EXTERN int		Tcl_MakeSafe _ANSI_ARGS_((Tcl_Interp *interp));
@@ -1351,7 +1456,8 @@ EXTERN int		Tcl_RecordAndEvalObj _ANSI_ARGS_((Tcl_Interp *interp,
 EXTERN Tcl_RegExp	Tcl_RegExpCompile _ANSI_ARGS_((Tcl_Interp *interp,
 			    char *string));
 EXTERN int		Tcl_RegExpExec _ANSI_ARGS_((Tcl_Interp *interp,
-			    Tcl_RegExp regexp, char *string, char *start));
+			    Tcl_RegExp regexp, CONST char *string,
+			    CONST char *start));
 EXTERN int		Tcl_RegExpMatch _ANSI_ARGS_((Tcl_Interp *interp,
 			    char *string, char *pattern));
 EXTERN void		Tcl_RegExpRange _ANSI_ARGS_((Tcl_RegExp regexp,
@@ -1402,6 +1508,9 @@ EXTERN void		Tcl_SetObjLength _ANSI_ARGS_((Tcl_Obj *objPtr,
 			    int length));
 EXTERN void		Tcl_SetObjResult _ANSI_ARGS_((Tcl_Interp *interp,
 			    Tcl_Obj *resultObjPtr));
+EXTERN Tcl_Obj *	Tcl_SetObjVar2 _ANSI_ARGS_((Tcl_Interp *interp,
+			    char *part1, char *part2, Tcl_Obj *newValuePtr,
+			    int flags));
 EXTERN void		Tcl_SetPanicProc _ANSI_ARGS_((void (*proc)
 			    _ANSI_ARGS_(TCL_VARARGS(char *, format))));
 EXTERN int		Tcl_SetRecursionLimit _ANSI_ARGS_((Tcl_Interp *interp,
@@ -1424,14 +1533,14 @@ EXTERN char *		Tcl_SignalMsg _ANSI_ARGS_((int sig));
 EXTERN void		Tcl_Sleep _ANSI_ARGS_((int ms));
 EXTERN void		Tcl_SourceRCFile _ANSI_ARGS_((Tcl_Interp *interp));
 EXTERN int		Tcl_SplitList _ANSI_ARGS_((Tcl_Interp *interp,
-			    char *list, int *argcPtr, char ***argvPtr));
-EXTERN void		Tcl_SplitPath _ANSI_ARGS_((char *path,
+			    CONST char *list, int *argcPtr, char ***argvPtr));
+EXTERN void		Tcl_SplitPath _ANSI_ARGS_((CONST char *path,
 			    int *argcPtr, char ***argvPtr));
 EXTERN void		Tcl_StaticPackage _ANSI_ARGS_((Tcl_Interp *interp,
 			    char *pkgName, Tcl_PackageInitProc *initProc,
 			    Tcl_PackageInitProc *safeInitProc));
-EXTERN int		Tcl_StringMatch _ANSI_ARGS_((char *string,
-			    char *pattern));
+EXTERN int		Tcl_StringMatch _ANSI_ARGS_((CONST char *string,
+			    CONST char *pattern));
 EXTERN int		Tcl_Tell _ANSI_ARGS_((Tcl_Channel chan));
 #define Tcl_TildeSubst Tcl_TranslateFileName
 EXTERN int		Tcl_TraceVar _ANSI_ARGS_((Tcl_Interp *interp,
@@ -1480,9 +1589,23 @@ EXTERN int		Tcl_WaitForEvent _ANSI_ARGS_((Tcl_Time *timePtr));
 EXTERN Tcl_Pid		Tcl_WaitPid _ANSI_ARGS_((Tcl_Pid pid, int *statPtr, 
 			    int options));
 EXTERN int		Tcl_Write _ANSI_ARGS_((Tcl_Channel chan,
-			    char *s, int slen));
+			    CONST char *src, int srcLen));
 EXTERN void		Tcl_WrongNumArgs _ANSI_ARGS_((Tcl_Interp *interp,
 			    int objc, Tcl_Obj *CONST objv[], char *message));
 
+/* Andreas Kupries <a.kupries@westend.com>, 05/31/1997.
+ * Support of Tcl-Trf.
+ */
+EXTERN Tcl_Channel      Tcl_ReplaceChannel _ANSI_ARGS_ ((Tcl_Interp* interp,
+                          Tcl_ChannelType* typePtr, ClientData instanceData,
+                          int mask, Tcl_Channel prevChan));
+
+EXTERN void             Tcl_UndoReplaceChannel _ANSI_ARGS_ ((Tcl_Interp* interp,
+                          Tcl_Channel chan));
+
 #endif /* RESOURCE_INCLUDED */
+
+#undef TCL_STORAGE_CLASS
+#define TCL_STORAGE_CLASS DLLIMPORT
+
 #endif /* _TCL */
