@@ -13,20 +13,12 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tclCmdIL.c 1.168 97/07/29 12:52:40
+ * SCCS: @(#) tclCmdIL.c 1.6 98/08/07 11:41:56
  */
 
 #include "tclInt.h"
 #include "tclPort.h"
-
-/*
- * The following variable holds the full path name of the binary
- * from which this application was executed, or NULL if it isn't
- * know.  The value of the variable is set by the procedure
- * Tcl_FindExecutable.  The storage space is dynamically allocated.
- */
-
-char *tclExecutableName = NULL;
+#include "tclCompile.h"
 
 /*
  * During execution of the "lsort" command, structures of the following
@@ -81,6 +73,9 @@ typedef struct SortInfo {
  * Forward declarations for procedures defined in this file:
  */
 
+static void		AppendLocals _ANSI_ARGS_((Tcl_Interp *interp,
+			    Tcl_Obj *listPtr, char *pattern,
+			    int includeLinks));
 static int		DictionaryCompare _ANSI_ARGS_((char *left,
 			    char *right));
 static int		InfoArgsCmd _ANSI_ARGS_((ClientData dummy,
@@ -131,6 +126,9 @@ static int		InfoPatchLevelCmd _ANSI_ARGS_((ClientData dummy,
 static int		InfoProcsCmd _ANSI_ARGS_((ClientData dummy,
 			    Tcl_Interp *interp, int objc,
 			    Tcl_Obj *CONST objv[]));
+static int		InfoResourceCmd _ANSI_ARGS_((ClientData dummy,
+			    Tcl_Interp *interp, int objc,
+			    Tcl_Obj *CONST objv[]));
 static int		InfoScriptCmd _ANSI_ARGS_((ClientData dummy,
 			    Tcl_Interp *interp, int objc,
 			    Tcl_Obj *CONST objv[]));
@@ -351,14 +349,14 @@ Tcl_InfoObjCmd(clientData, interp, objc, objv)
 	     "complete", "default", "exists", "globals",
 	     "hostname", "level", "library", "loaded",
 	     "locals", "nameofexecutable", "patchlevel", "procs",
-	     "script", "sharedlibextension", "tclversion", "vars",
+	     "resource", "script", "sharedlibextension", "tclversion", "vars",
 	     (char *) NULL};
     enum ISubCmdIdx {
 	    IArgsIdx, IBodyIdx, ICmdCountIdx, ICommandsIdx,
 	    ICompleteIdx, IDefaultIdx, IExistsIdx, IGlobalsIdx,
 	    IHostnameIdx, ILevelIdx, ILibraryIdx, ILoadedIdx,
 	    ILocalsIdx, INameOfExecutableIdx, IPatchLevelIdx, IProcsIdx,
-	    IScriptIdx, ISharedLibExtensionIdx, ITclVersionIdx, IVarsIdx
+	    IResourceIdx, IScriptIdx, ISharedLibExtensionIdx, ITclVersionIdx, IVarsIdx
     } index;
     int result;
 
@@ -422,6 +420,9 @@ Tcl_InfoObjCmd(clientData, interp, objc, objv)
         case IProcsIdx:
 	    result = InfoProcsCmd(clientData, interp, objc, objv);
 	    break;
+        case IResourceIdx:
+	    result = InfoResourceCmd(clientData, interp, objc, objv);
+	    break;
         case IScriptIdx:
 	    result = InfoScriptCmd(clientData, interp, objc, objv);
 	    break;
@@ -491,7 +492,7 @@ InfoArgsCmd(dummy, interp, objc, objv)
     listObjPtr = Tcl_NewListObj(0, (Tcl_Obj **) NULL);
     for (localPtr = procPtr->firstLocalPtr;  localPtr != NULL;
             localPtr = localPtr->nextPtr) {
-        if (localPtr->isArg) {
+        if (TclIsVarArgument(localPtr)) {
             Tcl_ListObjAppendElement(interp, listObjPtr,
 		    Tcl_NewStringObj(localPtr->name, -1));
         }
@@ -530,7 +531,8 @@ InfoBodyCmd(dummy, interp, objc, objv)
     register Interp *iPtr = (Interp *) interp;
     char *name;
     Proc *procPtr;
-
+    Tcl_Obj *bodyPtr, *resultPtr;
+    
     if (objc != 3) {
         Tcl_WrongNumArgs(interp, 2, objv, "procname");
         return TCL_ERROR;
@@ -543,7 +545,27 @@ InfoBodyCmd(dummy, interp, objc, objv)
 		"\"", name, "\" isn't a procedure", (char *) NULL);
         return TCL_ERROR;
     }
-    Tcl_SetObjResult(interp, procPtr->bodyPtr);
+
+    /*
+     * we need to check if the body from this procedure had been generated
+     * from a precompiled body. If that is the case, then the bodyPtr's
+     * string representation is bogus, since sources are not available.
+     * In order to make sure that later manipulations of the object do not
+     * invalidate the internal representation, we make a copy of the string
+     * representation and return that one, instead.
+     */
+
+    bodyPtr = procPtr->bodyPtr;
+    resultPtr = bodyPtr;
+    if (bodyPtr->typePtr == &tclByteCodeType) {
+        ByteCode *codePtr = (ByteCode *) bodyPtr->internalRep.otherValuePtr;
+
+        if (codePtr->flags & TCL_BYTECODE_PRECOMPILED) {
+            resultPtr = Tcl_NewStringObj(bodyPtr->bytes, bodyPtr->length);
+        }
+    }
+    
+    Tcl_SetObjResult(interp, resultPtr);
     return TCL_OK;
 }
 
@@ -813,7 +835,8 @@ InfoDefaultCmd(dummy, interp, objc, objv)
 
     for (localPtr = procPtr->firstLocalPtr;  localPtr != NULL;
             localPtr = localPtr->nextPtr) {
-        if ((localPtr->isArg) && (strcmp(argName, localPtr->name) == 0)) {
+        if (TclIsVarArgument(localPtr)
+		&& (strcmp(argName, localPtr->name) == 0)) {
             if (localPtr->defValuePtr != NULL) {
 		valueObjPtr = Tcl_ObjSetVar2(interp, objv[4], NULL,
                         localPtr->defValuePtr, 0);
@@ -987,13 +1010,21 @@ InfoHostnameCmd(dummy, interp, objc, objv)
     int objc;			/* Number of arguments. */
     Tcl_Obj *CONST objv[];	/* Argument objects. */
 {
+    char *name;
     if (objc != 2) {
         Tcl_WrongNumArgs(interp, 2, objv, NULL);
         return TCL_ERROR;
     }
 
-    Tcl_SetStringObj(Tcl_GetObjResult(interp), Tcl_GetHostName(), -1);
-    return TCL_OK;
+    name = Tcl_GetHostName();
+    if (name) {
+	Tcl_SetStringObj(Tcl_GetObjResult(interp), name, -1);
+	return TCL_OK;
+    } else {
+	Tcl_SetStringObj(Tcl_GetObjResult(interp),
+		"unable to determine name of host", -1);
+	return TCL_ERROR;
+    }
 }
 
 /*
@@ -1188,12 +1219,7 @@ InfoLocalsCmd(dummy, interp, objc, objv)
     Tcl_Obj *CONST objv[];	/* Argument objects. */
 {
     Interp *iPtr = (Interp *) interp;
-    Var *varPtr;
-    char *varName, *pattern;
-    int i, localVarCt;
-    Tcl_HashTable *localVarTablePtr;
-    register Tcl_HashEntry *entryPtr;
-    Tcl_HashSearch search;
+    char *pattern;
     Tcl_Obj *listPtr;
 
     if (objc == 2) {
@@ -1205,10 +1231,9 @@ InfoLocalsCmd(dummy, interp, objc, objv)
         return TCL_ERROR;
     }
     
-    if (iPtr->varFramePtr == NULL) {
+    if (iPtr->varFramePtr == NULL || !iPtr->varFramePtr->isProcCallFrame) {
         return TCL_OK;
     }
-    localVarTablePtr = iPtr->varFramePtr->varTablePtr;
 
     /*
      * Return a list containing names of first the compiled locals (i.e. the
@@ -1217,18 +1242,63 @@ InfoLocalsCmd(dummy, interp, objc, objv)
      */
     
     listPtr = Tcl_NewListObj(0, (Tcl_Obj **) NULL);
-    
+    AppendLocals(interp, listPtr, pattern, 0);
+    Tcl_SetObjResult(interp, listPtr);
+    return TCL_OK;
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * AppendLocals --
+ *
+ *	Append the local variables for the current frame to the
+ *	specified list object.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+static void
+AppendLocals(interp, listPtr, pattern, includeLinks)
+    Tcl_Interp *interp;		/* Current interpreter. */
+    Tcl_Obj *listPtr;		/* List object to append names to. */
+    char *pattern;		/* Pattern to match against. */
+    int includeLinks;		/* 1 if upvars should be included, else 0. */
+{
+    Interp *iPtr = (Interp *) interp;
+    CompiledLocal *localPtr;
+    Var *varPtr;
+    int i, localVarCt;
+    char *varName;
+    Tcl_HashTable *localVarTablePtr;
+    register Tcl_HashEntry *entryPtr;
+    Tcl_HashSearch search;
+
+    localPtr = iPtr->varFramePtr->procPtr->firstLocalPtr;
     localVarCt = iPtr->varFramePtr->numCompiledLocals;
-    for (i = 0, varPtr = iPtr->varFramePtr->compiledLocals;
-            i < localVarCt;
-	    i++, varPtr++) {
-	if (!TclIsVarUndefined(varPtr)) {
+    varPtr = iPtr->varFramePtr->compiledLocals;
+    localVarTablePtr = iPtr->varFramePtr->varTablePtr;
+
+    for (i = 0; i < localVarCt; i++) {
+	/*
+	 * Skip nameless (temporary) variables and undefined variables
+	 */
+
+	if (!TclIsVarTemporary(localPtr) && !TclIsVarUndefined(varPtr)) {
 	    varName = varPtr->name;
 	    if ((pattern == NULL) || Tcl_StringMatch(varName, pattern)) {
 		Tcl_ListObjAppendElement(interp, listPtr,
 		        Tcl_NewStringObj(varName, -1));
 	    }
         }
+	varPtr++;
+	localPtr = localPtr->nextPtr;
     }
     
     if (localVarTablePtr != NULL) {
@@ -1236,7 +1306,8 @@ InfoLocalsCmd(dummy, interp, objc, objv)
 	        entryPtr != NULL;
                 entryPtr = Tcl_NextHashEntry(&search)) {
 	    varPtr = (Var *) Tcl_GetHashValue(entryPtr);
-	    if (!TclIsVarUndefined(varPtr) && !TclIsVarLink(varPtr)) {
+	    if (!TclIsVarUndefined(varPtr)
+		    && (includeLinks || !TclIsVarLink(varPtr))) {
 		varName = Tcl_GetHashKey(localVarTablePtr, entryPtr);
 		if ((pattern == NULL)
 		        || Tcl_StringMatch(varName, pattern)) {
@@ -1246,9 +1317,6 @@ InfoLocalsCmd(dummy, interp, objc, objv)
 	    }
 	}
     }
-    
-    Tcl_SetObjResult(interp, listPtr);
-    return TCL_OK;
 }
 
 /*
@@ -1279,13 +1347,17 @@ InfoNameOfExecutableCmd(dummy, interp, objc, objv)
     int objc;			/* Number of arguments. */
     Tcl_Obj *CONST objv[];	/* Argument objects. */
 {
+    CONST char *nameOfExecutable;
+
     if (objc != 2) {
         Tcl_WrongNumArgs(interp, 2, objv, NULL);
         return TCL_ERROR;
     }
+
+    nameOfExecutable = Tcl_GetNameOfExecutable();
     
-    if (tclExecutableName != NULL) {
-	Tcl_SetStringObj(Tcl_GetObjResult(interp), tclExecutableName, -1);
+    if (nameOfExecutable != NULL) {
+	Tcl_SetStringObj(Tcl_GetObjResult(interp), (char *)nameOfExecutable, -1);
     }
     return TCL_OK;
 }
@@ -1400,6 +1472,59 @@ InfoProcsCmd(dummy, interp, objc, objv)
     return TCL_OK;
 }
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * InfoResourceCmd --
+ *
+ *      Called to implement the "info resource" command that returns the
+ *      content of the names resource. Handles the
+ *      following syntax:
+ *
+ *          info resource resourceName
+ *
+ * Results:
+ *      Returns TCL_OK is successful and TCL_ERROR is there is an error.
+ *
+ * Side effects:
+ *      Returns a result in the interpreter's result object. If there is
+ *	an error, the result is an error message.
+ *
+ *----------------------------------------------------------------------
+ */
+
+static int
+InfoResourceCmd(dummy, interp, objc, objv)
+    ClientData dummy;		/* Not used. */
+    Tcl_Interp *interp;		/* Current interpreter. */
+    int objc;			/* Number of arguments. */
+    Tcl_Obj *CONST objv[];	/* Argument objects. */
+{
+    char *resourceName;
+    char **value;
+
+    if (objc != 3) {
+        Tcl_WrongNumArgs(interp, 1, objv, "resource resourceName");
+        return TCL_ERROR;
+    }
+
+    resourceName = Tcl_GetStringFromObj(objv[2], (int *) NULL);
+    value = Tcl_GetStaticFile(resourceName);
+    if (value != (char **) NULL) {
+	char **p = value;
+	while (*p) {
+	    Tcl_AppendToObj(Tcl_GetObjResult(interp), *p, strlen(*p));
+	    Tcl_AppendToObj(Tcl_GetObjResult(interp), "\n", 1);
+	    p++;
+	}
+	return TCL_OK;
+    }
+    Tcl_AppendStringsToObj(Tcl_GetObjResult(interp),
+	    "no entry \"", resourceName, "\" in resource",
+	    (char *) NULL);
+    return TCL_ERROR;
+}
+
 /*
  *----------------------------------------------------------------------
  *
@@ -1557,13 +1682,13 @@ InfoVarsCmd(dummy, interp, objc, objv)
     char *varName, *pattern, *simplePattern;
     register Tcl_HashEntry *entryPtr;
     Tcl_HashSearch search;
-    Var *varPtr, *localVarPtr;
+    Var *varPtr;
     Namespace *nsPtr;
     Namespace *globalNsPtr = (Namespace *) Tcl_GetGlobalNamespace(interp);
     Namespace *currNsPtr   = (Namespace *) Tcl_GetCurrentNamespace(interp);
     Tcl_Obj *listPtr, *elemObjPtr;
     int specificNsInPattern = 0;  /* Init. to avoid compiler warning. */
-    int i, result;
+    int result;
 
     /*
      * Get the pattern and find the "effective namespace" in which to
@@ -1671,49 +1796,7 @@ InfoVarsCmd(dummy, interp, objc, objv)
 	    }
 	}
     } else {
-	/*
-	 * We're in a local call frame and no specific namespace was
-	 * specific. Create a list that starts with the compiled locals
-	 * (i.e. the ones stored in the call frame).
-	 */
-
-	CallFrame *varFramePtr = iPtr->varFramePtr;
-        int localVarCt = varFramePtr->numCompiledLocals;
-	Tcl_HashTable *varTablePtr = varFramePtr->varTablePtr;
-	
-        for (i = 0, localVarPtr = iPtr->varFramePtr->compiledLocals;
-                i < localVarCt;
-                i++, localVarPtr++) {
-            if (!TclIsVarUndefined(localVarPtr)) {
-                varName = localVarPtr->name;
-                if ((simplePattern == NULL)
-		        || Tcl_StringMatch(varName, simplePattern)) {
-                    Tcl_ListObjAppendElement(interp, listPtr,
-			    Tcl_NewStringObj(varName, -1));
-                }
-            }
-        }
-
-	/*
-	 * Now add in the variables in the call frame's variable hash
-	 * table (if one exists).
-	 */
-
-	if (varTablePtr != NULL) {
-	    for (entryPtr = Tcl_FirstHashEntry(varTablePtr, &search);
-		    entryPtr != NULL;
-		    entryPtr = Tcl_NextHashEntry(&search)) {
-		varPtr = (Var *) Tcl_GetHashValue(entryPtr);
-		if (!TclIsVarUndefined(varPtr)) {
-		    varName = Tcl_GetHashKey(varTablePtr, entryPtr);
-		    if ((simplePattern == NULL)
-		            || Tcl_StringMatch(varName, simplePattern)) {
-			Tcl_ListObjAppendElement(interp, listPtr,
-				Tcl_NewStringObj(varName, -1));
-		    }
-		}
-	    }
-	}
+	AppendLocals(interp, listPtr, simplePattern, 1);
     }
     
     Tcl_SetObjResult(interp, listPtr);
@@ -1748,6 +1831,7 @@ Tcl_JoinObjCmd(dummy, interp, objc, objv)
     char *joinString, *bytes;
     int joinLength, listLen, length, i, result;
     Tcl_Obj **elemPtrs;
+    Tcl_Obj *resObjPtr;
 
     if (objc == 2) {
 	joinString = " ";
@@ -1774,14 +1858,14 @@ Tcl_JoinObjCmd(dummy, interp, objc, objv)
      * directly into the interpreter's result object.
      */
 
+    resObjPtr = Tcl_GetObjResult(interp);
+
     for (i = 0;  i < listLen;  i++) {
 	bytes = Tcl_GetStringFromObj(elemPtrs[i], &length);
 	if (i > 0) {
-	    Tcl_AppendStringsToObj(Tcl_GetObjResult(interp), joinString,
-		    bytes, (char *) NULL);
-	} else {
-	    Tcl_AppendToObj(Tcl_GetObjResult(interp), bytes, length);
+	    Tcl_AppendToObj(resObjPtr, joinString, joinLength);
 	}
+	Tcl_AppendToObj(resObjPtr, bytes, length);
     }
     return TCL_OK;
 }
@@ -1895,8 +1979,8 @@ Tcl_LinsertObjCmd(dummy, interp, objc, objv)
     Tcl_Obj *CONST objv[];	/* Argument objects. */
 {
     Tcl_Obj *listPtr, *resultPtr;
-    int index, isDuplicate;
-    int result;
+    Tcl_ObjType *typePtr;
+    int index, isDuplicate, len, result;
 
     if (objc < 4) {
 	Tcl_WrongNumArgs(interp, 1, objv, "list index element ?element ...?");
@@ -1923,16 +2007,29 @@ Tcl_LinsertObjCmd(dummy, interp, objc, objv)
     listPtr = objv[1];
     isDuplicate = 0;
     if (Tcl_IsShared(listPtr)) {
+	/*
+	 * The following code must reflect the logic in Tcl_DuplicateObj()
+	 * except that it must duplicate the list object directly into the
+	 * interpreter's result.
+	 */
+	
 	Tcl_ResetResult(interp);
 	resultPtr = Tcl_GetObjResult(interp);
-	if (listPtr->typePtr != NULL) {
-	    Tcl_InvalidateStringRep(resultPtr);
-	    listPtr->typePtr->dupIntRepProc(listPtr, resultPtr);
-	} else if (listPtr->bytes != NULL) {
-	    int len = listPtr->length;
-	    
+	typePtr = listPtr->typePtr;
+	if (listPtr->bytes == NULL) {
+	    resultPtr->bytes = NULL;
+	} else if (listPtr->bytes != tclEmptyStringRep) {
+	    len = listPtr->length;
 	    TclInitStringRep(resultPtr, listPtr->bytes, len);
 	}
+	if (typePtr != NULL) {
+	    if (typePtr->dupIntRepProc == NULL) {
+		resultPtr->internalRep = listPtr->internalRep;
+		resultPtr->typePtr = typePtr;
+	    } else {
+		(*typePtr->dupIntRepProc)(listPtr, resultPtr);
+	    }
+	}
 	listPtr = resultPtr;
 	isDuplicate = 1;
     }
@@ -2164,7 +2261,9 @@ Tcl_LreplaceObjCmd(dummy, interp, objc, objv)
     Tcl_Obj *CONST objv[];	/* Argument objects. */
 {
     register Tcl_Obj *listPtr;
-    int createdNewObj, first, last, listLen, numToDelete, result;
+    int createdNewObj, first, last, listLen, numToDelete;
+    int firstArgLen, result;
+    char *firstArg;
 
     if (objc < 4) {
 	Tcl_WrongNumArgs(interp, 1, objv,
@@ -2201,6 +2300,7 @@ Tcl_LreplaceObjCmd(dummy, interp, objc, objv)
     if (result != TCL_OK) {
 	goto errorReturn;
     }
+    firstArg = Tcl_GetStringFromObj(objv[2], &firstArgLen);
 
     result = TclGetIntForIndex(interp, objv[3], /*endValue*/ (listLen - 1),
 	    &last);
@@ -2211,7 +2311,8 @@ Tcl_LreplaceObjCmd(dummy, interp, objc, objv)
     if (first < 0)  {
     	first = 0;
     }
-    if (first >= listLen) {
+    if ((first >= listLen) && (listLen > 0)
+	    && (strncmp(firstArg, "end", (unsigned) firstArgLen) != 0)) {
 	Tcl_AppendStringsToObj(Tcl_GetObjResult(interp),
 		"list doesn't contain element ",
 		Tcl_GetStringFromObj(objv[2], (int *) NULL), (int *) NULL);
@@ -2821,11 +2922,11 @@ DictionaryCompare(left, right)
 	     */
 
 	    zeros = 0;
-	    while (*right == '0') {
+	    while ((*right == '0') && (*(right + 1) != '\0')) {
 		right++;
 		zeros--;
 	    }
-	    while (*left == '0') {
+	    while ((*left == '0') && (*(left + 1) != '\0')) {
 		left++;
 		zeros++;
 	    }
@@ -2843,7 +2944,7 @@ DictionaryCompare(left, right)
 	    diff = 0;
 	    while (1) {
 		if (diff == 0) {
-		    diff = *left - *right;
+		    diff = UCHAR(*left) - UCHAR(*right);
 		}
 		right++;
 		left++;
@@ -2867,17 +2968,17 @@ DictionaryCompare(left, right)
 	    }
 	    continue;
 	}
-        diff = *left - *right;
+        diff = UCHAR(*left) - UCHAR(*right);
         if (diff) {
             if (isupper(UCHAR(*left)) && islower(UCHAR(*right))) {
-                diff = tolower(*left) - *right;
+                diff = UCHAR(tolower(*left)) - UCHAR(*right);
                 if (diff) {
 		    return diff;
                 } else if (secondaryDiff == 0) {
 		    secondaryDiff = -1;
                 }
             } else if (isupper(UCHAR(*right)) && islower(UCHAR(*left))) {
-                diff = *left - tolower(UCHAR(*right));
+                diff = UCHAR(*left) - UCHAR(tolower(UCHAR(*right)));
                 if (diff) {
 		    return diff;
                 } else if (secondaryDiff == 0) {
