@@ -5,12 +5,12 @@
  *	differences between the Mac and Unix. It should be the only
  *	file that contains #ifdefs to handle different flavors of OS.
  *
- * Copyright (c) 1995-1996 Sun Microsystems, Inc.
+ * Copyright (c) 1995-1997 Sun Microsystems, Inc.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tclMacPort.h 1.68 96/10/10 10:11:50
+ * SCCS: @(#) tclMacPort.h 1.75 97/08/11 10:18:07
  */
 
 #ifndef _MACPORT
@@ -21,6 +21,7 @@
 #endif
 
 #include "tclErrno.h"
+#include <float.h>
 
 /* Includes */
 #ifdef THINK_C
@@ -40,7 +41,18 @@
 #elif defined(__MWERKS__)
 #   include <time.h>
 #   include <unistd.h>
-#   include <fcntl.h>
+/*
+ * The following definitions are usually found if fcntl.h.
+ * However, MetroWerks has screwed that file up a couple of times
+ * and all we need are the defines.
+ */
+#define O_RDWR	  0x0		/* open the file in read/write mode */
+#define O_RDONLY  0x1		/* open the file in read only mode */
+#define O_WRONLY  0x2		/* open the file in write only mode */
+#define O_APPEND  0x0100	/* open the file in append mode */
+#define O_CREAT	  0x0200	/* create the file if it doesn't exist */
+#define O_EXCL	  0x0400	/* if the file exists don't create it again */
+#define O_TRUNC	  0x0800	/* truncate the file after opening it */
 
 /*
  * MetroWerks stat.h file is rather weak.  The defines
@@ -79,7 +91,7 @@
 
 /* 
  * Defines used by access function.  This function is provided
- * by Mac Tcl as the function TclMacAccess.
+ * by Mac Tcl as the function TclpAccess.
  */
  
 #   define	F_OK		0	/* test for existence of file */
@@ -106,17 +118,6 @@
 #define WTERMSIG(status) (1)
 #define WSTOPSIG(status) (1)
 
-/*
- * These defines are for functions that are now obsolete.  The only
- * Tcl code that still uses then is not called by the Mac.  The interfaces
- * should go away soon.
- */
-#define TclOpenFile(fname, mode) ((Tcl_File)NULL)
-#define TclCloseFile(file) (-1)
-#define TclReadFile(file, shouldBlock, buf, toRead) (toRead)
-#define TclWriteFile(file, shouldBlock, buf, toWrite) (-1)
-#define TclSeekFile(file, offset, whence) (-1)
-
 /*
  * Define "NBBY" (number of bits per byte) if it's not already defined.
  */
@@ -184,9 +185,6 @@ extern char **environ;
 EXTERN int 	TclMacCreateEnv _ANSI_ARGS_((void));
 EXTERN int	strncasecmp _ANSI_ARGS_((CONST char *s1,
 			    CONST char *s2, size_t n));
-#if (defined(THINK_C) || defined(__MWERKS__))
-double		hypot(double x, double y);
-#endif
 
 /*
  * The following declarations belong in tclInt.h, but depend on platform
@@ -197,18 +195,17 @@ EXTERN struct tm *	TclpGetDate _ANSI_ARGS_((const time_t *tp,
 			    int useGMT));
 EXTERN size_t		TclStrftime _ANSI_ARGS_((char *s, size_t maxsize,
 			    const char *format, const struct tm *t));
-#define TclStrftime(s,m,f,t) (strftime((s),(m),(f),(t)))
+
+#define tzset()
+#define TclpGetPid(pid)	    ((unsigned long) (pid))
 
 /*
- * The following prototypes and defines replace the Macintosh version
- * of the POSIX functions "stat" and "access".  The various compilier 
- * vendors don't implement this function well nor consistantly.
+ * The following defines replace the Macintosh version of the POSIX
+ * functions "stat" and "access".  The various compilier vendors
+ * don't implement this function well nor consistantly.
  */
-EXTERN int TclMacStat _ANSI_ARGS_((char *path, struct stat *buf));
-#define stat(path, bufPtr) TclMacStat(path, bufPtr)
-#define lstat(path, bufPtr) TclMacStat(path, bufPtr)
-EXTERN int TclMacAccess _ANSI_ARGS_((const char *filename, int mode));
-#define access(path, mode) TclMacAccess(path, mode)
+#define lstat(path, bufPtr) TclStat(path, bufPtr)
+
 EXTERN FILE * TclMacFOpenHack _ANSI_ARGS_((const char *path,
 	const char *mode));
 #define fopen(path, mode) TclMacFOpenHack(path, mode)
@@ -225,6 +222,7 @@ EXTERN int	TclMacChmod(char *path, int mode);
  */
 #define TclCreateCommandChannel(out, in, err, num, pidPtr)	NULL
 #define TclClosePipeFile(x)
+#define TclPlatformAsyncMark(async)
 
 /*
  * These definitions force putenv & company to use the version
@@ -251,6 +249,12 @@ void	TclUnsetEnv(CONST char *name);
 
 #define TCL_SHLIB_EXT ".shlb"
 
+/*
+ * TclpFinalize is a noop on the Mac.
+ */
+
+#define TclpFinalize()
+
 /*
  * The following define should really be in tclInt.h, but tclInt.h does
  * not include tclPort.h, which includes the "struct stat" definition.
@@ -259,5 +263,7 @@ void	TclUnsetEnv(CONST char *name);
 EXTERN int              TclpSameFile _ANSI_ARGS_((char *file1, char *file2,
 			    struct stat *sourceStatBufPtr, 
 		            struct stat *destStatBufPtr)) ;
+EXTERN int TclpStat _ANSI_ARGS_ ((CONST char *path, struct stat *buf));
+EXTERN int TclpAccess _ANSI_ARGS_ ((CONST char *path, int mode));
 
 #endif /* _MACPORT */
