@@ -9,7 +9,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tclUnixFile.c 1.41 96/12/05 14:59:20
+ * SCCS: @(#) tclUnixFile.c 1.48 97/07/07 16:38:11
  */
 
 #include "tclInt.h"
@@ -40,40 +40,6 @@ extern pid_t waitpid _ANSI_ARGS_((pid_t pid, int *stat_loc, int options));
 static void	FreeCurrentDir _ANSI_ARGS_((ClientData clientData));
 static void	FreeExecutableName _ANSI_ARGS_((ClientData clientData));
 
-/*
- *----------------------------------------------------------------------
- *
- * Tcl_WaitPid --
- *
- *	Implements the waitpid system call on Unix systems.
- *
- * Results:
- *	Result of calling waitpid.
- *
- * Side effects:
- *	Waits for a process to terminate.
- *
- *----------------------------------------------------------------------
- */
-
-int
-Tcl_WaitPid(pid, statPtr, options)
-    int pid;
-    int *statPtr;
-    int options;
-{
-    int result;
-    pid_t real_pid;
-
-    real_pid = (pid_t) pid;
-    while (1) {
-	result = (int) waitpid(real_pid, statPtr, options);
-	if ((result != -1) || (errno != EINTR)) {
-	    return result;
-	}
-    }
-}
-
 /*
  *----------------------------------------------------------------------
  *
@@ -99,6 +65,7 @@ FreeCurrentDir(clientData)
     if (currentDir != (char *) NULL) {
         ckfree(currentDir);
         currentDir = (char *) NULL;
+        currentDirExitHandlerSet = 0;
     }
 }
 
@@ -202,10 +169,22 @@ TclGetCwd(interp)
             currentDirExitHandlerSet = 1;
             Tcl_CreateExitHandler(FreeCurrentDir, (ClientData) NULL);
         }
+#ifdef USEGETWD
+	if ((int)getwd(buffer) == (int)NULL) {
+	    if (interp != NULL) {
+		Tcl_AppendResult(interp,
+			"error getting working directory name: ",
+			buffer, (char *)NULL);
+	    }
+	    return NULL;
+	}
+#else
 	if (getcwd(buffer, MAXPATHLEN+1) == NULL) {
 	    if (interp != NULL) {
 		if (errno == ERANGE) {
-		    interp->result = "working directory name is too long";
+		    Tcl_SetResult(interp,
+			    "working directory name is too long",
+		            TCL_STATIC);
 		} else {
 		    Tcl_AppendResult(interp,
 			    "error getting working directory name: ",
@@ -214,239 +193,13 @@ TclGetCwd(interp)
 	    }
 	    return NULL;
 	}
+#endif
 	currentDir = (char *) ckalloc((unsigned) (strlen(buffer) + 1));
 	strcpy(currentDir, buffer);
     }
     return currentDir;
 }
 
-/*
- *----------------------------------------------------------------------
- *
- * TclOpenFile --
- *
- *	Implements a mechanism to open files on Unix systems.
- *
- * Results:
- *	The opened file.
- *
- * Side effects:
- *	May cause a file to be created on the file system.
- *
- *----------------------------------------------------------------------
- */
-
-Tcl_File
-TclOpenFile(fname, mode)
-    char *fname;			/* The name of the file to open. */
-    int mode;				/* In what mode to open the file? */
-{
-    int fd;
-
-    fd = open(fname, mode, 0666);
-    if (fd != -1) {
-        fcntl(fd, F_SETFD, FD_CLOEXEC);
-        return Tcl_GetFile((ClientData)fd, TCL_UNIX_FD);
-    }
-    return NULL;
-}
-
-/*
- *----------------------------------------------------------------------
- *
- * TclCloseFile --
- *
- *	Implements a mechanism to close a UNIX file.
- *
- * Results:
- *	Returns 0 on success, or -1 on error, setting errno.
- *
- * Side effects:
- *	The file is closed.
- *
- *----------------------------------------------------------------------
- */
-
-int
-TclCloseFile(file)
-    Tcl_File file;	/* The file to close. */
-{
-    int type;
-    int fd;
-    int result;
-
-    fd = (int) Tcl_GetFileInfo(file, &type);
-    if (type != TCL_UNIX_FD) {
-	panic("Tcl_CloseFile: unexpected file type");
-    }
-
-    /*
-     * Refuse to close the fds for stdin, stdout and stderr.
-     */
-    
-    if ((fd == 0) || (fd == 1) || (fd == 2)) {
-        return 0;
-    }
-    
-    result = close(fd);
-    Tcl_DeleteFileHandler(file);
-    Tcl_FreeFile(file);
-    return result;
-}
-
-/*
- *----------------------------------------------------------------------
- *
- * TclReadFile --
- *
- *	Implements a mechanism to read from files on Unix systems. Also
- *	simulates blocking behavior on non-blocking files when asked to.
- *
- * Results:
- *	The number of characters read from the specified file.
- *
- * Side effects:
- *	May consume characters from the file.
- *
- *----------------------------------------------------------------------
- */
-	/* ARGSUSED */
-int
-TclReadFile(file, shouldBlock, buf, toRead)
-    Tcl_File file;	/* The file to read from. */
-    int shouldBlock;		/* Not used. */
-    char *buf;			/* The buffer to store input in. */
-    int toRead;			/* Number of characters to read. */
-{
-    int type, fd;
-
-    fd = (int) Tcl_GetFileInfo(file, &type);
-    if (type != TCL_UNIX_FD) {
-	panic("Tcl_ReadFile: unexpected file type");
-    }
-
-    return read(fd, buf, (size_t) toRead);
-}
-
-/*
- *----------------------------------------------------------------------
- *
- * TclWriteFile --
- *
- *	Implements a mechanism to write to files on Unix systems.
- *
- * Results:
- *	The number of characters written to the specified file.
- *
- * Side effects:
- *	May produce characters on the file.
- *
- *----------------------------------------------------------------------
- */
-
-	/* ARGSUSED */
-int
-TclWriteFile(file, shouldBlock, buf, toWrite)
-    Tcl_File file;	/* The file to write to. */
-    int shouldBlock;		/* Not used. */
-    char *buf;			/* Where output is stored. */
-    int toWrite;		/* Number of characters to write. */
-{
-    int type, fd;
-
-    fd = (int) Tcl_GetFileInfo(file, &type);
-    if (type != TCL_UNIX_FD) {
-	panic("Tcl_WriteFile: unexpected file type");
-    }
-    return write(fd, buf, (size_t) toWrite);
-}
-
-/*
- *----------------------------------------------------------------------
- *
- * TclSeekFile --
- *
- *	Sets the file pointer on the indicated UNIX file.
- *
- * Results:
- *	The new position at which the file will be accessed, or -1 on
- *	failure.
- *
- * Side effects:
- *	May change the position at which subsequent operations access the
- *	file designated by the file.
- *
- *----------------------------------------------------------------------
- */
-
-int
-TclSeekFile(file, offset, whence)
-    Tcl_File file;	/* The file to seek on. */
-    int offset;			/* How far to seek? */
-    int whence;			/* And from where to seek? */
-{
-    int type, fd;
-
-    fd = (int) Tcl_GetFileInfo(file, &type);
-    if (type != TCL_UNIX_FD) {
-	panic("Tcl_SeekFile: unexpected file type");
-    }
-
-    return lseek(fd, offset, whence);
-}
-
-/*
- *----------------------------------------------------------------------
- *
- * TclCreateTempFile --
- *
- *	This function creates a temporary file initialized with an
- *	optional string, and returns a file handle with the file pointer
- *	at the beginning of the file.
- *
- * Results:
- *	A handle to a file.
- *
- * Side effects:
- *	None.
- *
- *----------------------------------------------------------------------
- */
-
-Tcl_File
-TclCreateTempFile(contents, namePtr)
-    char *contents;		/* String to write into temp file, or NULL. */
-    Tcl_DString *namePtr;	/* If non-NULL, pointer to initialized 
-				 * DString that is filled with the name of 
-				 * the temp file that was created. */
-{
-    char fileName[L_tmpnam];
-    Tcl_File file;
-    size_t length = (contents == NULL) ? 0 : strlen(contents);
-
-    tmpnam(fileName);
-    file = TclOpenFile(fileName, O_RDWR|O_CREAT|O_TRUNC);
-    unlink(fileName);
-
-    if ((file != NULL) && (length > 0)) {
-	int fd = (int)Tcl_GetFileInfo(file, NULL);
-	while (1) {
-	    if (write(fd, contents, length) != -1) {
-		break;
-	    } else if (errno != EINTR) {
-		close(fd);
-		Tcl_FreeFile(file);
-		return NULL;
-	    }
-	}
-	lseek(fd, 0, SEEK_SET);
-    }
-    if (namePtr != NULL) {
-	Tcl_DStringAppend(namePtr, fileName, -1);
-    }
-    return file;
-}
-
 /*
  *----------------------------------------------------------------------
  *
@@ -468,11 +221,12 @@ TclCreateTempFile(contents, namePtr)
 
 void
 Tcl_FindExecutable(argv0)
-    char *argv0;		/* The value of the application's argv[0]. */
+    CONST char *argv0;		/* The value of the application's argv[0]. */
 {
-    char *name, *p, *cwd;
+    CONST char *name, *p, *cwd;
     Tcl_DString buffer;
     int length;
+    struct stat statBuf;
 
     Tcl_DStringInit(&buffer);
     if (tclExecutableName != NULL) {
@@ -500,6 +254,12 @@ Tcl_FindExecutable(argv0)
 	 */
 
 	p = ":/bin:/usr/bin";
+    } else if (*p == '\0') {
+	/*
+	 * An empty path is equivalent to ".".
+	 */
+
+	p = "./";
     }
 
     /*
@@ -524,11 +284,19 @@ Tcl_FindExecutable(argv0)
 	    }
 	}
 	Tcl_DStringAppend(&buffer, argv0, -1);
-	if (access(Tcl_DStringValue(&buffer), X_OK) == 0) {
+	if ((TclAccess(Tcl_DStringValue(&buffer), X_OK) == 0)
+		&& (TclStat(Tcl_DStringValue(&buffer), &statBuf) == 0)
+		&& S_ISREG(statBuf.st_mode)) {
 	    name = Tcl_DStringValue(&buffer);
 	    goto gotName;
 	}
-	p++;
+	if (*p == 0) {
+	    break;
+	} else if (*(p+1) == 0) {
+	    p = "./";
+	} else {
+	    p++;
+	}
     }
     goto done;
 
@@ -632,7 +400,8 @@ TclGetUserHome(name, bufferPtr)
  * Side effects:
  *	None.
  *
- *---------------------------------------------------------------------- */
+ *----------------------------------------------------------------------
+ */
 
 int
 TclMatchFiles(interp, separators, dirPtr, pattern, tail)
@@ -665,7 +434,7 @@ TclMatchFiles(interp, separators, dirPtr, pattern, tail)
     } else {
 	dirName = dirPtr->string;
     }
-    if ((stat(dirName, &statBuf) != 0) || !S_ISDIR(statBuf.st_mode)) {
+    if ((TclStat(dirName, &statBuf) != 0) || !S_ISDIR(statBuf.st_mode)) {
 	return TCL_OK;
     }
 
@@ -751,7 +520,7 @@ TclMatchFiles(interp, separators, dirPtr, pattern, tail)
 	    Tcl_DStringAppend(dirPtr, entryPtr->d_name, -1);
 	    if (tail == NULL) {
 		Tcl_AppendElement(interp, dirPtr->string);
-	    } else if ((stat(dirPtr->string, &statBuf) == 0)
+	    } else if ((TclStat(dirPtr->string, &statBuf) == 0)
 		    && S_ISDIR(statBuf.st_mode)) {
 		Tcl_DStringAppend(dirPtr, "/", 1);
 		result = TclDoGlob(interp, separators, dirPtr, tail);
