@@ -8,12 +8,20 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tclUnixTest.c 1.4 97/05/14 13:24:29
+ * SCCS: @(#) tclUnixTest.c 1.5 97/10/31 17:23:42
  */
 
 #include "tclInt.h"
 #include "tclPort.h"
 
+/*
+ * The headers are needed for the testalarm command that verifies the
+ * use of SA_RESTART in signal handlers.
+ */
+
+#include <signal.h>
+#include <sys/resource.h>
+
 /*
  * The following macros convert between TclFile's and fd's.  The conversion
  * simple involves shifting fd's up by one to ensure that no valid fd is ever
@@ -28,6 +36,10 @@
  * exercised by the "testfilehandler" command.
  */
 
+#ifdef __cplusplus
+#   define dummy /* */
+#endif
+
 typedef struct Pipe {
     TclFile readFile;		/* File handle for reading from the
 				 * pipe.  NULL means pipe doesn't exist yet. */
@@ -44,6 +56,12 @@ typedef struct Pipe {
 #define MAX_PIPES 10
 static Pipe testPipes[MAX_PIPES];
 
+/*
+ * The stuff below is used by the testalarm and testgotsig ommands.
+ */
+
+static char *gotsig = "0";
+
 /*
  * Forward declarations of procedures defined later in this file:
  */
@@ -54,9 +72,17 @@ static int		TestfilehandlerCmd _ANSI_ARGS_((ClientData dummy,
 			    Tcl_Interp *interp, int argc, char **argv));
 static int		TestfilewaitCmd _ANSI_ARGS_((ClientData dummy,
 			    Tcl_Interp *interp, int argc, char **argv));
+static int		TestfindexecutableCmd _ANSI_ARGS_((ClientData dummy,
+			    Tcl_Interp *interp, int argc, char **argv));
 static int		TestgetopenfileCmd _ANSI_ARGS_((ClientData dummy,
 			    Tcl_Interp *interp, int argc, char **argv));
 int			TclplatformtestInit _ANSI_ARGS_((Tcl_Interp *interp));
+int                   Unixtest_Init _ANSI_ARGS_((Tcl_Interp *interp));
+static int		TestalarmCmd _ANSI_ARGS_((ClientData dummy,
+			    Tcl_Interp *interp, int argc, char **argv));
+static int		TestgotsigCmd _ANSI_ARGS_((ClientData dummy,
+			    Tcl_Interp *interp, int argc, char **argv));
+static void 		AlarmHandler _ANSI_ARGS_(());
 
 /*
  *----------------------------------------------------------------------
@@ -76,15 +102,61 @@ int			TclplatformtestInit _ANSI_ARGS_((Tcl_Interp *interp));
  */
 
 int
+#ifdef _USING_PROTOTYPES_
+TclplatformtestInit (
+    Tcl_Interp *interp)		/* Interpreter to add commands to. */
+#else
 TclplatformtestInit(interp)
     Tcl_Interp *interp;		/* Interpreter to add commands to. */
+#endif
 {
+    return Unixtest_Init(interp);
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * Unixtest_Init --
+ *
+ *	Defines commands that test platform specific functionality for
+ *	Unix platforms.
+ *
+ * Results:
+ *	A standard Tcl result.
+ *
+ * Side effects:
+ *	Defines new commands.
+ *
+ *----------------------------------------------------------------------
+ */
+
+int
+#ifdef _USING_PROTOTYPES_
+Unixtest_Init (
+    Tcl_Interp *interp)		/* Interpreter to add commands to. */
+#else
+Unixtest_Init(interp)
+    Tcl_Interp *interp;		/* Interpreter to add commands to. */
+#endif
+{
+    if (Tcl_PkgRequire(interp, "Tcl", TCL_VERSION, 1) == NULL) {
+	return TCL_ERROR;
+    }
+    if (Tcl_PkgProvide(interp, "Unixtest", TCL_VERSION) != TCL_OK) {
+        return TCL_ERROR;
+    }
     Tcl_CreateCommand(interp, "testfilehandler", TestfilehandlerCmd,
             (ClientData) 0, (Tcl_CmdDeleteProc *) NULL);
     Tcl_CreateCommand(interp, "testfilewait", TestfilewaitCmd,
             (ClientData) 0, (Tcl_CmdDeleteProc *) NULL);
+    Tcl_CreateCommand(interp, "testfindexecutable", TestfindexecutableCmd,
+            (ClientData) 0, (Tcl_CmdDeleteProc *) NULL);
     Tcl_CreateCommand(interp, "testgetopenfile", TestgetopenfileCmd,
             (ClientData) 0, (Tcl_CmdDeleteProc *) NULL);
+    Tcl_CreateCommand(interp, "testalarm", TestalarmCmd,
+            (ClientData) 0, (Tcl_CmdDeleteProc *) NULL);
+    Tcl_CreateCommand(interp, "testgotsig", TestgotsigCmd,
+            (ClientData) 0, (Tcl_CmdDeleteProc *) NULL);
     return TCL_OK;
 }
 
@@ -107,11 +179,19 @@ TclplatformtestInit(interp)
  */
 
 static int
-TestfilehandlerCmd(clientData, interp, argc, argv)
-    ClientData clientData;		/* Not used. */
+#ifdef _USING_PROTOTYPES_
+TestfilehandlerCmd (
+    ClientData dummy,			/* Not used. */
+    Tcl_Interp *interp,			/* Current interpreter. */
+    int argc,				/* Number of arguments. */
+    char **argv)			/* Argument strings. */
+#else
+TestfilehandlerCmd(dummy, interp, argc, argv)
+    ClientData dummy;			/* Not used. */
     Tcl_Interp *interp;			/* Current interpreter. */
     int argc;				/* Number of arguments. */
     char **argv;			/* Argument strings. */
+#endif
 {
     Pipe *pipePtr;
     int i, mask, timeout;
@@ -301,10 +381,18 @@ TestfilehandlerCmd(clientData, interp, argc, argv)
     return TCL_OK;
 }
 
-static void TestFileHandlerProc(clientData, mask)
+static void
+#ifdef _USING_PROTOTYPES_
+TestFileHandlerProc (
+    ClientData clientData,	/* Points to a Pipe structure. */
+    int mask)			/* Indicates which events happened:
+				 * TCL_READABLE or TCL_WRITABLE. */
+#else
+TestFileHandlerProc(clientData, mask)
     ClientData clientData;	/* Points to a Pipe structure. */
     int mask;			/* Indicates which events happened:
 				 * TCL_READABLE or TCL_WRITABLE. */
+#endif
 {
     Pipe *pipePtr = (Pipe *) clientData;
 
@@ -343,6 +431,7 @@ TestfilewaitCmd(clientData, interp, argc, argv)
     int mask, result, timeout;
     Tcl_Channel channel;
     int fd;
+    ClientData data;
 
     if (argc != 4) {
 	Tcl_AppendResult(interp, "wrong # arguments: should be \"", argv[0],
@@ -366,10 +455,11 @@ TestfilewaitCmd(clientData, interp, argc, argv)
     }
     if (Tcl_GetChannelHandle(channel, 
 	    (mask & TCL_READABLE) ? TCL_READABLE : TCL_WRITABLE,
-	    (ClientData*) &fd) != TCL_OK) {
+	    (ClientData*) &data) != TCL_OK) {
 	Tcl_SetResult(interp, "couldn't get channel file", TCL_STATIC);
 	return TCL_ERROR;
     }
+    fd = (int) data;
     if (Tcl_GetInt(interp, argv[3], &timeout) != TCL_OK) {
 	return TCL_ERROR;
     }
@@ -383,6 +473,48 @@ TestfilewaitCmd(clientData, interp, argc, argv)
     return TCL_OK;
 }
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * TestfindexecutableCmd --
+ *
+ *	This procedure implements the "testfindexecutable" command. It is
+ *	used to test Tcl_FindExecutable.
+ *
+ * Results:
+ *	A standard Tcl result.
+ *
+ * Side effects:
+ *	None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+static int
+TestfindexecutableCmd(clientData, interp, argc, argv)
+    ClientData clientData;		/* Not used. */
+    Tcl_Interp *interp;			/* Current interpreter. */
+    int argc;				/* Number of arguments. */
+    char **argv;			/* Argument strings. */
+{
+    char *oldName;
+
+    if (argc != 2) {
+	Tcl_AppendResult(interp, "wrong # arguments: should be \"", argv[0],
+		" argv0\"", (char *) NULL);
+	return TCL_ERROR;
+    }
+    oldName = tclExecutableName;
+    tclExecutableName = NULL;
+    Tcl_FindExecutable(argv[1]);
+    if (tclExecutableName != NULL) {
+	Tcl_SetResult(interp, tclExecutableName, TCL_VOLATILE);
+	ckfree(tclExecutableName);
+    }
+    tclExecutableName = oldName;
+    return TCL_OK;
+}
+
 /*
  *----------------------------------------------------------------------
  *
@@ -401,11 +533,19 @@ TestfilewaitCmd(clientData, interp, argc, argv)
  */
 
 static int
-TestgetopenfileCmd(clientData, interp, argc, argv)
-    ClientData clientData;		/* Not used. */
+#ifdef _USING_PROTOTYPES_
+TestgetopenfileCmd (
+    ClientData dummy,			/* Not used. */
+    Tcl_Interp *interp,			/* Current interpreter. */
+    int argc,				/* Number of arguments. */
+    char **argv)			/* Argument strings. */
+#else
+TestgetopenfileCmd(dummy, interp, argc, argv)
+    ClientData dummy;			/* Not used. */
     Tcl_Interp *interp;			/* Current interpreter. */
     int argc;				/* Number of arguments. */
     char **argv;			/* Argument strings. */
+#endif
 {
     ClientData filePtr;
 
@@ -427,3 +567,109 @@ TestgetopenfileCmd(clientData, interp, argc, argv)
     }
     return TCL_OK;
 }
+
+/*
+ *----------------------------------------------------------------------
+ * TestalarmCmd --
+ *
+ *	Test that EINTR is handled correctly by generating and
+ *	handling a signal.  This requires using the SA_RESTART
+ *	flag when registering the signal handler.
+ *
+ * Results:
+ *	None.
+ *
+ * Side Effects:
+ *	Sets up an signal and async handlers.
+ *
+ *----------------------------------------------------------------------
+ */
+
+int
+TestalarmCmd(clientData, interp, argc, argv)
+    ClientData clientData;		/* Not used. */
+    Tcl_Interp *interp;			/* Current interpreter. */
+    int argc;				/* Number of arguments. */
+    char **argv;			/* Argument strings. */
+{
+#ifdef SA_RESTART
+    unsigned int sec;
+    struct sigaction action;
+
+    if (argc > 1) {
+	Tcl_GetInt(interp, argv[1], (int *)&sec);
+    } else {
+	sec = 1;
+    }
+
+    /*
+     * Setup the signal handling that automatically retries
+     * any interupted I/O system calls.
+     */
+    action.sa_handler = AlarmHandler;
+    memset((void *)&action.sa_mask, 0, sizeof(sigset_t));
+    action.sa_flags = SA_RESTART;
+
+    if (sigaction(SIGALRM, &action, NULL) < 0) {
+	Tcl_AppendResult(interp, "sigaction: ", Tcl_PosixError(interp), NULL);
+	return TCL_ERROR;
+    }
+    if (alarm(sec) < 0) {
+	Tcl_AppendResult(interp, "alarm: ", Tcl_PosixError(interp), NULL);
+	return TCL_ERROR;
+    }
+    return TCL_OK;
+#else
+    Tcl_AppendResult(interp, "warning: sigaction SA_RESTART not support on this platform", NULL);
+    return TCL_ERROR;
+#endif
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * AlarmHandler --
+ *
+ *	Signal handler for the alarm command.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ * 	Calls the Tcl Async handler.
+ *
+ *----------------------------------------------------------------------
+ */
+
+static void
+AlarmHandler()
+{
+    gotsig = "1";
+}
+
+/*
+ *----------------------------------------------------------------------
+ * TestgotsigCmd --
+ *
+ * 	Verify the signal was handled after the testalarm command.
+ *
+ * Results:
+ *	None.
+ *
+ * Side Effects:
+ *	Resets the value of gotsig back to '0'.
+ *
+ *----------------------------------------------------------------------
+ */
+
+int
+TestgotsigCmd(clientData, interp, argc, argv)
+    ClientData clientData;		/* Not used. */
+    Tcl_Interp *interp;			/* Current interpreter. */
+    int argc;				/* Number of arguments. */
+    char **argv;			/* Argument strings. */
+{
+    Tcl_AppendResult(interp, gotsig, (char *) NULL);
+    gotsig = "0";
+    return TCL_OK;
+}
