@@ -9,7 +9,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tclWinPipe.c 1.44 97/08/05 11:46:12
+ * SCCS: @(#) tclWinPipe.c 1.49 97/11/06 17:33:03
  */
 
 #include "tclWinInt.h"
@@ -26,6 +26,13 @@
 
 static int initialized = 0;
 
+/*
+ * The following define declares a new user message for use on the
+ * pipe window.
+ */
+
+#define PIPE_MESSAGE	WM_USER+20
+
 /*
  * The following defines identify the various types of applications that 
  * run under windows.  There is special case code for the various types.
@@ -95,7 +102,7 @@ typedef struct ProcInfo {
     struct ProcInfo *nextPtr;
 } ProcInfo;
 
-static ProcInfo *procList;
+static ProcInfo *procList = NULL;
 
 /*
  * State flags used in the PipeInfo structure below.
@@ -122,6 +129,10 @@ typedef struct PipeInfo {
     TclFile errorFile;		/* Error output from pipe. */
     int numPids;		/* Number of processes attached to pipe. */
     Tcl_Pid *pidPtr;		/* Pids of attached processes. */
+    TclWinReaderInfo *readerInfo; /* File reader state */
+    int refCount;		/* Reference count for free routine */
+
+    struct PipeInfo *nextLPtr;	/* Pointer to next created pipe */ 
     struct PipeInfo *nextPtr;	/* Pointer to next registered pipe. */
 } PipeInfo;
 
@@ -130,7 +141,13 @@ typedef struct PipeInfo {
  * that are being watched for file events.
  */
 
-static PipeInfo *firstPipePtr;
+static PipeInfo *firstPipePtr = NULL;
+
+/*
+ * Every open pipe has an entry on the following list.
+ */
+
+static PipeInfo *pipeList = NULL;
 
 /*
  * The following structure is what is added to the Tcl event queue when
@@ -146,12 +163,25 @@ typedef struct PipeEvent {
 				 * pointer. */
 } PipeEvent;
 
+/*
+ * Pipe notification window.  This window is used to receive pipe
+ * notification events.
+ */
+
+static HWND pipeWindow = NULL;
+
+/*
+ * Window class for creating the pipe notification window.
+ */
+
+static ATOM pipeClass;
+
 /*
  * Declarations for functions used only in this file.
  */
 
 static int	ApplicationType(Tcl_Interp *interp, const char *fileName,
-		    char *fullName);
+		    char *fullName, char *imagePath);
 static void	BuildCommandLine(int argc, char **argv, Tcl_DString *linePtr);
 static void	CopyChannel(HANDLE dst, HANDLE src);
 static BOOL	HasConsole(void);
@@ -163,6 +193,7 @@ static void	PipeCheckProc _ANSI_ARGS_((ClientData clientData,
 static int	PipeCloseProc(ClientData instanceData, Tcl_Interp *interp);
 static int	PipeEventProc(Tcl_Event *evPtr, int flags);
 static void	PipeExitHandler(ClientData clientData);
+static void	PipeFreeProc(ClientData clientData);
 static int	PipeGetHandleProc(ClientData instanceData, int direction,
 		    ClientData *handlePtr);
 static void	PipeInit(void);
@@ -170,9 +201,13 @@ static int	PipeInputProc(ClientData instanceData, char *buf, int toRead,
 		    int *errorCode);
 static int	PipeOutputProc(ClientData instanceData, char *buf, int toWrite,
 		    int *errorCode);
+static LRESULT CALLBACK PipeProc (HWND hwnd, UINT message, WPARAM wParam,
+		    LPARAM lParam);
 static void	PipeWatchProc(ClientData instanceData, int mask);
-static void	PipeSetupProc _ANSI_ARGS_((ClientData clientData,
-		    int flags));
+static int	PipeSetOptionProc (ClientData instanceData, Tcl_Interp *interp,
+		    char *optionName, char *value);
+static void	PipeSetupProc(ClientData clientData, int flags);
+static int	TempFileName(char name[MAX_PATH]);
 
 /*
  * This structure describes the channel type structure for command pipe
@@ -186,7 +221,7 @@ static Tcl_ChannelType pipeChannelType = {
     PipeInputProc,		/* Input proc. */
     PipeOutputProc,		/* Output proc. */
     NULL,			/* Seek proc. */
-    NULL,			/* Set option proc. */
+    PipeSetOptionProc,		/* Set option proc. */
     NULL,			/* Get option proc. */
     PipeWatchProc,		/* Set up notifier to watch the channel. */
     PipeGetHandleProc,		/* Get an OS handle from channel. */
@@ -211,13 +246,97 @@ static Tcl_ChannelType pipeChannelType = {
 static void
 PipeInit()
 {
+    WNDCLASS class;
+
+    if (initialized) {
+	return;
+    }
+
     initialized = 1;
-    firstPipePtr = NULL;
-    procList = NULL;
+
+    /*
+     * Register the async notification window class and window.
+     */
+
+    class.style = 0;
+    class.cbClsExtra = 0;
+    class.cbWndExtra = 0;
+    class.hInstance = TclWinGetTclInstance();
+    class.hbrBackground = NULL;
+    class.lpszMenuName = NULL;
+    class.lpszClassName = "TclPipe";
+    class.lpfnWndProc = PipeProc;
+    class.hIcon = NULL;
+    class.hCursor = NULL;
+
+    pipeClass = RegisterClass(&class);
+    if (!pipeClass) {
+	return;
+    }
+    pipeWindow = CreateWindowEx(0, (LPCTSTR)pipeClass, "TclPipe",
+	    WS_OVERLAPPED, 0, 0, 0, 0, NULL, NULL,
+	    TclWinGetTclInstance(), NULL);
+    if (pipeWindow == NULL) {
+	TclWinConvertError(GetLastError());
+	UnregisterClass((LPCTSTR)pipeClass, TclWinGetTclInstance());
+	return;
+    }
+
     Tcl_CreateEventSource(PipeSetupProc, PipeCheckProc, NULL);
     Tcl_CreateExitHandler(PipeExitHandler, NULL);
 }
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * PipeProc --
+ *
+ *	This function is called when WSAAsyncSelect has been used
+ *	to register interest in a pipe event, and the event has
+ *	occurred.
+ *
+ * Results:
+ *	0 on success.
+ *
+ * Side effects:
+ *	The flags for the given pipe are updated to reflect the
+ *	event that occured.
+ *
+ *----------------------------------------------------------------------
+ */
+
+static LRESULT CALLBACK
+PipeProc(hwnd, message, wParam, lParam)
+    HWND hwnd;
+    UINT message;
+    WPARAM wParam;
+    LPARAM lParam;
+{
+    int event;
+    HANDLE pipe;
+    PipeInfo *pipePtr;
+
+    if ((hwnd != pipeWindow) || (message != PIPE_MESSAGE)) {
+	return DefWindowProc(hwnd, message, wParam, lParam);
+    }
+    
+    pipe = (HANDLE) wParam;
+    event = lParam;
+
+    /*
+     * Find the specified pipe on the pipe list and update its
+     * check flags.
+     */
+
+    for (pipePtr = pipeList; pipePtr != NULL; pipePtr = pipePtr->nextLPtr) {
+	if (pipePtr->readerInfo->rHandle == pipe) {
+	    pipePtr->readerInfo->readyMask |= event;
+	    break;
+	}
+    }
+    return 0;
+}
+
 /*
  *----------------------------------------------------------------------
  *
@@ -239,6 +358,8 @@ static void
 PipeExitHandler(clientData)
     ClientData clientData;	/* Old window proc */
 {
+    DestroyWindow(pipeWindow);
+    UnregisterClass((LPCTSTR)pipeClass, TclWinGetTclInstance());
     Tcl_DeleteEventSource(PipeSetupProc, PipeCheckProc, NULL);
     initialized = 0;
 }
@@ -273,11 +394,22 @@ PipeSetupProc(data, flags)
     }
     
     /*
-     * Check to see if there is a watched pipe.  If so, poll.
+     * Check to see if there is a watched pipe.  If so, poll on Win32s.
      */
 
     for (infoPtr = firstPipePtr; infoPtr != NULL; infoPtr = infoPtr->nextPtr) {
-	if (infoPtr->watchMask) {
+	if ((infoPtr->watchMask & TCL_READABLE) &&
+	    (infoPtr->readerInfo->rType != FILE_TYPE_DISK))
+	{
+	    if (infoPtr->readerInfo->rThread == NULL) {
+		TclWinReaderStart(infoPtr->readerInfo, TclWinReaderThread);
+	    }
+	}
+	if ((infoPtr->watchMask & TCL_WRITABLE) ||
+	    ((infoPtr->readerInfo->readyMask & infoPtr->watchMask)
+	     && ((infoPtr->readerInfo->rDataLen > infoPtr->readerInfo->rDataPos)
+		 || infoPtr->readerInfo->eof)))
+	{
 	    Tcl_SetMaxBlockTime(&blockTime);
 	    break;
 	}
@@ -319,7 +451,19 @@ PipeCheckProc(data, flags)
      */
 
     for (infoPtr = firstPipePtr; infoPtr != NULL; infoPtr = infoPtr->nextPtr) {
-	if (infoPtr->watchMask && !(infoPtr->flags & PIPE_PENDING)) {
+	if ((infoPtr->watchMask & TCL_READABLE) &&
+	    (infoPtr->readerInfo->rType != FILE_TYPE_DISK))
+	{
+	    if (infoPtr->readerInfo->rThread == NULL) {
+		TclWinReaderStart(infoPtr->readerInfo, TclWinReaderThread);
+	    }
+	}
+	if ((infoPtr->watchMask & TCL_WRITABLE) ||
+	    (((infoPtr->readerInfo->readyMask) & infoPtr->watchMask)
+	     && (infoPtr->readerInfo->eof ||
+		 ((infoPtr->readerInfo->rDataLen > infoPtr->readerInfo->rDataPos)
+		  && !(infoPtr->flags & PIPE_PENDING)))))
+	{
 	    infoPtr->flags |= PIPE_PENDING;
 	    evPtr = (PipeEvent *) ckalloc(sizeof(PipeEvent));
 	    evPtr->header.proc = PipeEventProc;
@@ -359,6 +503,28 @@ MakeFile(handle)
     return (TclFile)filePtr;
 }
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * TclWinMakeFile --
+ *
+ *	Exported version of MakeFile().
+ *
+ * Results:
+ *	Returns a newly allocated WinFile as a TclFile.
+ *
+ * Side effects:
+ *	None.
+ *
+ *----------------------------------------------------------------------
+ */
+TclFile
+TclWinMakeFile(handle)
+    HANDLE handle;		/* Type-specific data. */
+{
+    return MakeFile(handle);
+}
+
 /*
  *----------------------------------------------------------------------
  *
@@ -390,6 +556,43 @@ TclpMakeFile(channel, direction)
     }
 }
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * TempFileName --
+ *
+ *	Gets a temporary file name and deals with the fact that the
+ *	temporary file path provided by Windows may not actually exist
+ *	if the TMP or TEMP environment variables refer to a 
+ *	non-existent directory.
+ *
+ * Results:    
+ *	0 if error, non-zero otherwise.  If non-zero is returned, the
+ *	name buffer will be filled with a name that can be used to 
+ *	construct a temporary file.
+ *
+ * Side effects:
+ *	None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+static int
+TempFileName(name)
+    char name[MAX_PATH];	/* Buffer in which name for temporary 
+				 * file gets stored. */
+{
+    if ((GetTempPath(MAX_PATH, name) == 0) ||
+	    (GetTempFileName(name, "TCL", 0, name) == 0)) {
+	name[0] = '.';
+	name[1] = '\0';
+	if (GetTempFileName(name, "TCL", 0, name) == 0) {
+	    return 0;
+	}
+    }
+    return 1;
+}
+
 /*
  *----------------------------------------------------------------------
  *
@@ -418,8 +621,7 @@ TclpCreateTempFile(contents, namePtr)
     char name[MAX_PATH];
     HANDLE handle;
 
-    if ((GetTempPath(MAX_PATH, name) == 0) ||
-	    (GetTempFileName(name, "TCL", 0, name) == 0)) {
+    if (TempFileName(name) == 0) {
 	return NULL;
     }
 
@@ -648,9 +850,7 @@ TclpCreatePipe(readPipe, writePipe)
 	WinPipe *readPipePtr, *writePipePtr;
 	char buf[MAX_PATH];
 
-	if ((GetTempPath(MAX_PATH, buf) != 0)
-	        && (GetTempFileName(buf, "TCL", 0, buf) != 0)) {
-
+	if (TempFileName(buf) != 0) {
 	    readPipePtr = (WinPipe *) ckalloc(sizeof(WinPipe));
 	    writePipePtr = (WinPipe *) ckalloc(sizeof(WinPipe));
 
@@ -836,10 +1036,15 @@ TclpCreateProcess(interp, argc, argv, inputFile, outputFile, errorFile,
     SECURITY_ATTRIBUTES secAtts;
     HANDLE hProcess, h, inputHandle, outputHandle, errorHandle;
     char execPath[MAX_PATH];
+    char image[MAX_PATH];
     char *originalName;
     WinFile *filePtr;
 
-    applType = ApplicationType(interp, argv[0], execPath);
+    if (!initialized) {
+	PipeInit();
+    }
+
+    applType = ApplicationType(interp, argv[0], execPath, &image[0]);
     if (applType == APPL_NONE) {
 	return TCL_ERROR;
     }
@@ -1236,6 +1441,10 @@ TclpCreateProcess(interp, argc, argv, inputFile, outputFile, errorFile,
      * using ab~1.def instead of "a b.default").  
      */
 
+    if (image[0] != '\0') {
+	Tcl_DStringAppend(&cmdLine, image, strlen(image));	
+	Tcl_DStringAppend(&cmdLine, " ", 1);	
+    }
     BuildCommandLine(argc, argv, &cmdLine);
 
     if (!CreateProcess(NULL, Tcl_DStringValue(&cmdLine), NULL, NULL, TRUE, 
@@ -1355,11 +1564,12 @@ HasConsole()
  */
 
 static int
-ApplicationType(interp, originalName, fullPath)
+ApplicationType(interp, originalName, fullPath, imagePath)
     Tcl_Interp *interp;		/* Interp, for error message. */
     const char *originalName;	/* Name of the application to find. */
     char fullPath[MAX_PATH];	/* Filled with complete path to 
 				 * application. */
+    char *imagePath;
 {
     int applType, i;
     HANDLE hFile;
@@ -1382,6 +1592,9 @@ ApplicationType(interp, originalName, fullPath)
      * the extensions, looking for a match.  
      */
 
+    if (imagePath) {
+	imagePath[0] = '\0';
+    }
     applType = APPL_NONE;
     for (i = 0; i < (int) (sizeof(extensions) / sizeof(extensions[0])); i++) {
 	lstrcpyn(fullPath, originalName, MAX_PATH - 5);
@@ -1398,6 +1611,28 @@ ApplicationType(interp, originalName, fullPath)
 	    continue;
 	}
 
+	/*
+	 * MKS shell needs to be handled like a DOS program, otherwise
+	 * it start acting wacky...  Basically, it doesn't like being
+	 * detached.
+	 * XXX: We need to have a better way of determining that this
+	 * is an MKS shell such as looking inside the resources.
+	 * XXX: This needs to be case insensitive.
+	 */
+	if (strstr(fullPath, "mks") || strstr(fullPath, "MKS")) {
+	    char *p = strrchr(fullPath, '\\');
+	    if (!p) {
+		p = strrchr(fullPath, '/');
+	    }
+	    if (p) {
+		p++;
+		if (stricmp(p, "sh.exe")) {
+		    applType = APPL_DOS;
+		    break;
+		}
+	    }
+	}
+
 	ext = strrchr(fullPath, '.');
 	if ((ext != NULL) && (strcmpi(ext, ".bat") == 0)) {
 	    applType = APPL_DOS;
@@ -1412,6 +1647,66 @@ ApplicationType(interp, originalName, fullPath)
 
 	header.e_magic = 0;
 	ReadFile(hFile, (void *) &header, sizeof(header), &read, NULL);
+
+	/*
+	 * A bit of a hack.  Look for a '#!' at the start of the file.  If we
+	 * see it, it indicates the presence of a shell script or something
+	 * along those lines (i.e. perl, tcl, etc).  We extract the name of the
+	 * image that it is looking for, and we try and turn it into the name
+	 * of the Win32 image that needs to be run.
+	 */
+	if (header.e_magic == 0x2123 ) { /* #! */
+	    char *cpnt;
+	    char *dpnt;
+	    char scriptName[MAX_PATH];
+	    char shellPath[MAX_PATH];
+	    char tmpBuffer[MAX_PATH];
+	    Tcl_DString execBuffer;
+
+	    buf[0] = '\0';
+	    SetFilePointer(hFile, 0, NULL, FILE_BEGIN);
+	    ReadFile(hFile, (void *) tmpBuffer, MAX_PATH, &read, NULL);
+	    /*
+	     * Extract the name of the script interpreter.
+	     */
+	    cpnt = tmpBuffer + 2;
+	    while (isspace(*cpnt) && *cpnt != '\n') {
+		cpnt++;
+	    }
+	    dpnt = scriptName;
+	    
+	    while (*cpnt && !isspace(*cpnt)) {
+		*dpnt++ = *cpnt++;
+	    }
+	    *dpnt = '\0';
+	    
+	    /*
+	     * First try and normalize the name to a Win32 name.
+	     */
+	    Tcl_DStringInit(&execBuffer);
+	    Tcl_TranslateFileName(interp, scriptName, &execBuffer);
+	    applType = ApplicationType(interp, scriptName, shellPath, NULL);
+	    
+	    if (applType == APPL_NONE) {
+		if (strcmp(scriptName, "/bin/sh") == 0) {
+		    cpnt = getenv("SHELL");
+		    if (cpnt == NULL) {
+			continue;
+		    }
+		    cpnt = Tcl_TranslateFileName(interp, cpnt, &execBuffer);
+		    strcpy(scriptName, cpnt);
+		    applType = ApplicationType(interp, scriptName, shellPath, NULL);
+		}
+	    }
+	    if (applType != APPL_NONE && imagePath != NULL) {
+		strcpy(imagePath, shellPath);
+	    }
+
+	    CloseHandle(hFile);
+	    Tcl_DStringFree(&execBuffer);
+	    return applType;
+	}
+
 	if (header.e_magic != IMAGE_DOS_SIGNATURE) {
 	    /* 
 	     * Doesn't have the magic number for relocatable executables.  If 
@@ -1449,22 +1744,35 @@ ApplicationType(interp, originalName, fullPath)
 	ReadFile(hFile, (void *) buf, 2, &read, NULL);
 	CloseHandle(hFile);
 
-	if ((buf[0] == 'L') && (buf[1] == 'E')) {
-	    applType = APPL_DOS;
-	} else if ((buf[0] == 'N') && (buf[1] == 'E')) {
+	if ((buf[0] == 'N') && (buf[1] == 'E')) {
 	    applType = APPL_WIN3X;
 	} else if ((buf[0] == 'P') && (buf[1] == 'E')) {
 	    applType = APPL_WIN32;
 	} else {
-	    continue;
+	    /*
+	     * Strictly speaking, there should be a test that there
+	     * is an 'L' and 'E' at buf[0..1], to identify the type as 
+	     * DOS, but of course we ran into a DOS executable that 
+	     * _doesn't_ have the magic number -- specifically, one
+	     * compiled using the Lahey Fortran90 compiler.
+	     */
+
+	    applType = APPL_DOS;
 	}
 	break;
     }
 
     if (applType == APPL_NONE) {
-	TclWinConvertError(GetLastError());
-	Tcl_AppendResult(interp, "couldn't execute \"", originalName,
-		"\": ", Tcl_PosixError(interp), (char *) NULL);
+	/*
+	 * If we have a NULL imagePath, it means we are searching for the command
+	 * interpreter.  We don't record the errors here, since we may
+	 * have to look at several possibilities.
+	 */
+	if (imagePath != NULL) {
+	    TclWinConvertError(GetLastError());
+	    Tcl_AppendResult(interp, "couldn't execute \"", originalName,
+			     "\": ", Tcl_PosixError(interp), (char *) NULL);
+	}
 	return APPL_NONE;
     }
 
@@ -1516,13 +1824,19 @@ BuildCommandLine(argc, argv, linePtr)
 	}
 
 	quote = 0;
-	for (start = argv[i]; *start != '\0'; start++) {
-	    if (isspace(*start)) {
-		quote = 1;
-		Tcl_DStringAppend(linePtr, "\"", 1);
-    		break;
+	if (argv[i][0] == '\0') {
+	    quote = 1;
+	} else {
+	    for (start = argv[i]; *start != '\0'; start++) {
+		if (isspace(*start)) {
+		    quote = 1;
+		    break;
+		}
 	    }
 	}
+	if (quote) {
+	    Tcl_DStringAppend(linePtr, "\"", 1);
+	}
 
 	start = argv[i];	    
 	for (special = argv[i]; ; ) {
@@ -1593,8 +1907,7 @@ MakeTempFile(namePtr)
 {
     char name[MAX_PATH];
 
-    if ((GetTempPath(MAX_PATH, name) == 0)
-	    || (GetTempFileName(name, "TCL", 0, name) == 0)) {
+    if (TempFileName(name) == 0) {
 	return NULL;
     }
 
@@ -1671,6 +1984,10 @@ TclpCreateCommandChannel(readFile, writeFile, errorFile, numPids, pidPtr)
     int channelId;
     PipeInfo *infoPtr = (PipeInfo *) ckalloc((unsigned) sizeof(PipeInfo));
 
+    if (!initialized) {
+	PipeInit();
+    }
+
     infoPtr->watchMask = 0;
     infoPtr->flags = 0;
     infoPtr->readFile = readFile;
@@ -1678,6 +1995,17 @@ TclpCreateCommandChannel(readFile, writeFile, errorFile, numPids, pidPtr)
     infoPtr->errorFile = errorFile;
     infoPtr->numPids = numPids;
     infoPtr->pidPtr = pidPtr;
+    infoPtr->refCount = 1;
+
+    if (!initialized) {
+	PipeInit();
+    }
+    if (readFile) {
+	infoPtr->readerInfo = TclWinReaderNew(pipeWindow, PIPE_MESSAGE,
+	    ((WinFile*)infoPtr->readFile)->handle);
+    } else {
+	infoPtr->readerInfo = NULL;
+    }
 
     /*
      * Use one of the fds associated with the channel as the
@@ -1728,6 +2056,21 @@ TclpCreateCommandChannel(readFile, writeFile, errorFile, numPids, pidPtr)
 	    "-translation", "auto");
     Tcl_SetChannelOption((Tcl_Interp *) NULL, infoPtr->channel,
 	    "-eofchar", "\032 {}");
+
+    if (readFile) {
+	infoPtr->refCount++;
+    }
+    if (writeFile) {
+	infoPtr->refCount++;
+    }
+    if (errorFile) {
+	infoPtr->refCount++;
+    }
+
+    infoPtr->nextPtr = NULL;
+    infoPtr->nextLPtr = pipeList;
+    pipeList = infoPtr;
+
     return infoPtr->channel;
 }
 
@@ -1808,12 +2151,19 @@ PipeBlockModeProc(instanceData, mode)
      * hence we have to emulate the behavior. This is done in the input
      * function by checking against a bit in the state. We set or unset the
      * bit here to cause the input function to emulate the correct behavior.
+     * If readerInfo is not set, the pipe was only opened for writing
      */
 
     if (mode == TCL_MODE_NONBLOCKING) {
 	infoPtr->flags |= PIPE_ASYNC;
+	if (infoPtr->readerInfo) {
+	    infoPtr->readerInfo->async = 1;
+	}
     } else {
 	infoPtr->flags &= ~(PIPE_ASYNC);
+	if (infoPtr->readerInfo) {
+	    infoPtr->readerInfo->async = 0;
+	}
     }
     return 0;
 }
@@ -1843,9 +2193,10 @@ PipeCloseProc(instanceData, interp)
     Tcl_Channel errChan;
     int errorCode, result;
     PipeInfo *infoPtr, **nextPtrPtr;
+    HANDLE rSemaphore = NULL;
 
     /*
-     * Remove the file from the list of watched files.
+     * Remove the pipe from the list of watched pipes.
      */
 
     for (nextPtrPtr = &firstPipePtr, infoPtr = *nextPtrPtr; infoPtr != NULL;
@@ -1856,20 +2207,18 @@ PipeCloseProc(instanceData, interp)
 	}
     }
 
-    errorCode = 0;
-    if (pipePtr->readFile != NULL) {
-	if (TclpCloseFile(pipePtr->readFile) != 0) {
-	    errorCode = errno;
+    /*
+     * Remove the pipe from the list of all pipes.
+     */
+
+    for (nextPtrPtr = &pipeList, infoPtr = *nextPtrPtr; infoPtr != NULL;
+	    nextPtrPtr = &infoPtr->nextLPtr, infoPtr = *nextPtrPtr) {
+	if (infoPtr == (PipeInfo *)pipePtr) {
+	    *nextPtrPtr = infoPtr->nextLPtr;
+	    break;
 	}
     }
-    if (pipePtr->writeFile != NULL) {
-	if (TclpCloseFile(pipePtr->writeFile) != 0) {
-	    if (errorCode == 0) {
-		errorCode = errno;
-	    }
-	}
-    }
-    
+
     /*
      * Wrap the error file into a channel and give it to the cleanup
      * routine.  If we are running in Win32s, just delete the error file
@@ -1890,21 +2239,134 @@ PipeCloseProc(instanceData, interp)
 	    errChan = Tcl_MakeFileChannel((ClientData) filePtr->handle,
 		    TCL_READABLE);
 	}
+	pipePtr->refCount--;
     } else {
         errChan = NULL;
     }
+    errorCode = 0;
+    if (pipePtr->readFile != NULL) {
+	if (TclpCloseFile(pipePtr->readFile) != 0) {
+	    errorCode = errno;
+	}
+	pipePtr->readerInfo->rHandleClosed = TRUE;
+	pipePtr->refCount--;
+    }
+
+    if (pipePtr->writeFile != NULL) {
+	if (!pipePtr->readFile ||
+	    (((WinFile *) pipePtr->writeFile)->handle !=
+	     ((WinFile *) pipePtr->readFile)->handle)) {
+	    if (TclpCloseFile(pipePtr->writeFile) != 0) {
+		if (errorCode == 0) {
+		    errorCode = errno;
+		}
+	    }
+	}
+	pipePtr->refCount--;
+    }
     result = TclCleanupChildren(interp, pipePtr->numPids, pipePtr->pidPtr,
             errChan);
-    if (pipePtr->numPids > 0) {
-        ckfree((char *) pipePtr->pidPtr);
-    }
-    ckfree((char*) pipePtr);
+
+    pipePtr->refCount--;
+    PipeFreeProc((ClientData) pipePtr);
 
     if (errorCode == 0) {
         return result;
     }
     return errorCode;
 }
+/*
+ *----------------------------------------------------------------------
+ *
+ * PipeFreeProc --
+ *
+ *	This callback is invoked by Tcl_FreeFile in order to delete
+ *	the notifier data associated with a file handle.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Removes the PipeInfo from the global pipe list.
+ *
+ *----------------------------------------------------------------------
+ */
+
+static void
+PipeFreeProc(clientData)
+    ClientData clientData;
+{
+    PipeInfo *pipePtr = (PipeInfo *) clientData;
+
+    if (--pipePtr->refCount > 0) {
+	return;
+    }
+    if (pipePtr->readerInfo) {
+	TclWinReaderFree(pipePtr->readerInfo);
+    }
+    if (pipePtr->numPids > 0) {
+        ckfree((char *) pipePtr->pidPtr);
+    }
+    ckfree((char *) pipePtr);
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * PipeSetOptionProc --
+ *
+ *	Sets an option on a channel.  Used internally to set 
+ *	
+ *
+ * Results:
+ *	A standard Tcl result. Also sets interp->result on error if
+ *	interp is not NULL.
+ *
+ * Side effects:
+ *	May modify an option on a device.
+ *
+ *----------------------------------------------------------------------
+ */
+
+static int		
+PipeSetOptionProc(instanceData, interp, optionName, value)
+    ClientData instanceData;	/* File state. */
+    Tcl_Interp *interp;		/* For error reporting - can be NULL. */
+    char *optionName;		/* Which option to set? */
+    char *value;		/* New value for option. */
+{
+    PipeInfo *pipePtr;
+    HANDLE handle;
+    int len;
+
+    pipePtr = (PipeInfo *) instanceData;
+    handle = (HANDLE) ((WinFile*)pipePtr->readFile)->handle;
+
+    if (handle == NULL) {
+	Tcl_AppendResult(interp, "not opened for reading", NULL);
+	return TCL_ERROR;
+    }
+
+    len = strlen(optionName);
+    if ((len > 1) && (strncmp(optionName, "-iomode", len) == 0)) {
+	len = strlen(value);
+	if (len > 1) {
+	    if (strncmp(value, "overlapped", len) == 0) {
+		pipePtr->readerInfo->overlappedIO = 1;
+	    } else if (strncmp(value, "synchronous", len) == 0) {
+		pipePtr->readerInfo->overlappedIO = 0;
+	    } else {
+                if (interp != NULL) {
+                    Tcl_AppendResult(interp,
+			"bad value for -iomode: should be one of ",
+			"overlapped or synchronous", NULL);
+		    return TCL_ERROR;
+		}
+	    }
+	}
+    }
+    return TCL_OK;
+}
 
 /*
  *----------------------------------------------------------------------
@@ -1934,7 +2396,6 @@ PipeInputProc(instanceData, buf, bufSize, errorCode)
 {
     PipeInfo *infoPtr = (PipeInfo *) instanceData;
     WinFile *filePtr = (WinFile*) infoPtr->readFile;
-    DWORD count;
     DWORD bytesRead;
 
     *errorCode = 0;
@@ -1948,58 +2409,23 @@ PipeInputProc(instanceData, buf, bufSize, errorCode)
 		    NULL);
 	}
 	if (filePtr->handle == INVALID_HANDLE_VALUE) {
-	    goto error;
+	    goto win32s_error;
 	}
-    } else {
-	/*
-	 * Pipes will block until the requested number of bytes has been
-	 * read.  To avoid blocking unnecessarily, we look ahead and only
-	 * read as much as is available.
-	 */
-
-	if (PeekNamedPipe(filePtr->handle, (LPVOID) NULL, (DWORD) 0,
-		(LPDWORD) NULL, &count, (LPDWORD) NULL) == TRUE) {
-	    if ((count != 0) && ((DWORD) bufSize > count)) {
-		bufSize = (int) count;
-
-		/*
-		 * This code is commented out because on Win95 we don't get
-		 * notifier of eof on a pipe unless we try to read it.
-		 * The correct solution is to move to threads.
-		 */
-
-/* 	    } else if ((count == 0) && (infoPtr->flags & PIPE_ASYNC)) { */
-/* 		errno = *errorCode = EAGAIN; */
-/* 		return -1; */
-	    } else if ((count == 0) && !(infoPtr->flags & PIPE_ASYNC)) {
-		bufSize = 1;
-	    }
-	} else {
-	    goto error;
-	}
-    }
-
-    /*
-     * Note that we will block on reads from a console buffer until a
-     * full line has been entered.  The only way I know of to get
-     * around this is to write a console driver.  We should probably
-     * do this at some point, but for now, we just block.
-     */
-
-    if (ReadFile(filePtr->handle, (LPVOID) buf, (DWORD) bufSize, &bytesRead,
+	if (ReadFile(filePtr->handle, (LPVOID) buf, (DWORD) bufSize, &bytesRead,
             (LPOVERLAPPED) NULL) == FALSE) {
-	goto error;
-    }
+	    goto win32s_error;
+	}
     
-    return bytesRead;
-
-    error:
-    TclWinConvertError(GetLastError());
-    if (errno == EPIPE) {
-	return 0;
+	return bytesRead;
+    win32s_error:
+	TclWinConvertError(GetLastError());
+	if (errno == EPIPE) {
+	    return 0;
+	}
+	*errorCode = errno;
+	return -1;
     }
-    *errorCode = errno;
-    return -1;
+    return TclWinReaderInput(infoPtr->readerInfo, buf, bufSize, errorCode);
 }
 
 /*
@@ -2035,9 +2461,6 @@ PipeOutputProc(instanceData, buf, toWrite, errorCode)
     if (WriteFile(filePtr->handle, (LPVOID) buf, (DWORD) toWrite,
 	    &bytesWritten, (LPOVERLAPPED) NULL) == FALSE) {
         TclWinConvertError(GetLastError());
-        if (errno == EPIPE) {
-            return 0;
-        }
         *errorCode = errno;
         return -1;
     }
@@ -2174,10 +2597,6 @@ PipeWatchProc(instanceData, mask)
     PipeInfo *infoPtr = (PipeInfo *) instanceData;
     int oldMask = infoPtr->watchMask;
 
-    if (!initialized) {
-	PipeInit();
-    }
-
     /*
      * For now, we just send a message to ourselves so we can poll the
      * channel for readable events.
@@ -2289,11 +2708,14 @@ Tcl_WaitPid(pid, statPtr, options)
     Tcl_Pid result;
     DWORD ret;
 
-    if (options & WNOHANG) {
-	flags = 0;
-    } else {
-	flags = INFINITE;
+    if (!initialized) {
+	PipeInit();
     }
+
+    /*
+     * If no pid is specified, do nothing.
+     */
+    
     if (pid == 0) {
 	*statPtr = 0;
 	return 0;
@@ -2310,10 +2732,27 @@ Tcl_WaitPid(pid, statPtr, options)
 	    break;
 	}
     }
+
+    /*
+     * If the pid is not one of the processes we know about (we started it)
+     * then do nothing.
+     */
+    
     if (infoPtr == NULL) {
+        *statPtr = 0;
 	return 0;
     }
 
+    /*
+     * Officially "wait" for it to finish. We either poll (WNOHANG) or
+     * wait for an infinite amount of time.
+     */
+    
+    if (options & WNOHANG) {
+	flags = 0;
+    } else {
+	flags = INFINITE;
+    }
     ret = WaitForSingleObject(infoPtr->hProcess, flags);
     if (ret == WAIT_TIMEOUT) {
 	*statPtr = 0;
@@ -2328,12 +2767,14 @@ Tcl_WaitPid(pid, statPtr, options)
 	result = pid;
     } else {
 	errno = ECHILD;
+        *statPtr = ECHILD;
 	result = (Tcl_Pid) -1;
     }
 
     /*
      * Remove the process from the process list and close the process handle.
      */
+
     CloseHandle(infoPtr->hProcess);
     *prevPtrPtr = infoPtr->nextPtr;
     ckfree((char*)infoPtr);
@@ -2341,6 +2782,32 @@ Tcl_WaitPid(pid, statPtr, options)
     return result;
 }
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * TclWinAddProcess --
+ *
+ *	Add a process to the process list so that we can use
+ *	Tcl_WaitPid on the process.
+ *
+ * Results:
+ *	None
+ *
+ *----------------------------------------------------------------------
+ */
+
+void
+TclWinAddProcess(hProcess, id)
+    HANDLE hProcess;		/* Handle to process */
+    DWORD id;			/* Global process identifier */
+{
+    ProcInfo *procPtr = (ProcInfo *) ckalloc(sizeof(ProcInfo));
+    procPtr->hProcess = hProcess;
+    procPtr->dwProcessId = id;
+    procPtr->nextPtr = procList;
+    procList = procPtr;
+}
+
 /*
  *----------------------------------------------------------------------
  *
